import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.io.*;
import java.util.*;


/**
 * This class builts the component which is popped-up when selecting the Replace characters option in the Settings menu. It is possible to use the default set for replacemnts or to built a new one.
 * @see UserOptions
 */
public class ReplaceCharacters implements ActionListener,UserOptions{
    /**
     * the user's set of strings to be replaced 
     */
    CharactersToBeReplaced userMask;
    //the widgets
    JTextField from_string;
    JTextField to_string;
    JTextArea list;
    JButton addList;
    JButton removeList;
    JButton default_button;
    JButton dismiss;
    JButton write_default;
    JLabel change_string;
    JDialog dialog;
    

    JFrame mainframe;
    /**
     * referrecnce to the User interface
     */
    UserInterface ui;
    /**
     * the model for the table which shows the strings to be replaced
     */
    DefaultTableModel myModel;
    /**
     * the table
     */
    JTable table;

   
    /**
     * builts all the widgets and add them to the window
     * builts also the objects containing the default mask and the users mask
     * @param mainframe the parent window
     * @param ui referrence to the user interface
     */
    ReplaceCharacters(JFrame mainFrame, UserInterface ui){
	this.mainframe=mainframe;
	this.ui=ui;

	//initially the user mask is the default mask
	userMask = new CharactersToBeReplaced(new DefaultCharactersToBeReplaced());
	//builts the widgets
	from_string=new JTextField(10);
	to_string=new JTextField(10);
	change_string=new JLabel("change in");
	JPanel nord=new JPanel(new FlowLayout());
	nord.add(from_string);
	nord.add(change_string);
	nord.add(to_string);

	// the table
	final String[] columnNames={"From","To"};
	myModel = new DefaultTableModel(userMask.getData(),columnNames);
	table = new JTable(myModel);
	table.setEnabled(false);
	table.setPreferredScrollableViewportSize(new Dimension(100,100));
	JScrollPane scrollPane=new JScrollPane(table);

	//the buttons
	addList=new JButton("Add");
	addList.addActionListener(this);
	removeList=new JButton("Remove");
	removeList.setEnabled(true);
	removeList.addActionListener(this);
	default_button=new JButton("Default");
	default_button.addActionListener(this);

	write_default=new JButton("Save Default");
	write_default.addActionListener(this);

	dismiss = new JButton("Close");
	dismiss.addActionListener(this);
	JPanel buttons = new JPanel(new GridLayout(5,1));
	buttons.add(addList);
	buttons.add(removeList);
	buttons.add(default_button);
	buttons.add(write_default);
	buttons.add(dismiss);
	JPanel center = new JPanel(new BorderLayout());
	center.add(scrollPane,BorderLayout.CENTER);
	center.add(buttons, BorderLayout.WEST);

	//adds control to the window
	dialog = new JDialog(mainframe,"Replace characters");
	dialog.addWindowListener(new WindowAdapter(){
		public void windowClosing(WindowEvent e){
		    dialog.hide();
		}
	    });

	//adds the componenets to the window and packs them
	Container c=dialog.getContentPane();
	c.setLayout(new BorderLayout());
	c.add(nord,BorderLayout.NORTH);
	c.add(center,BorderLayout.CENTER);
	dialog.pack();
    }

    /**
     * @return the users's mask
     */
    public String[][] getData() {
	return userMask.getData();
    }

      
    /**
     * pops-up the window
     */
    public void ShowDialog() {
	dialog.show();
    }

    /**
     * dismiss the window
     */
    public void HideDialog() {
	dialog.hide();
    }


    /**
     * describes actions on the widgets
     * @see CharactersToBeReplaced#getData()
     * @see CharactersToBeReplaced#setData(int)
     * @see #getSelection(JTable)
     */
    public void actionPerformed(ActionEvent e) {
	JButton b= (JButton) e.getSource();
	String data;
	int pos = 0;

       
	if(b== addList) {
	    //collects the strings from the textfields
	    String s1= from_string.getText();
	    String s2=to_string.getText();
	    //if there is something to be replaced
	    if (!s1.equals("")) {
		//retrive the number of replacements in the user mask
		pos=userMask.getMembersNumber();
		//if there is at least one row free
		if (pos < REPLACEMASKSIZE) {
		    //increase the number of strings to be replaced in the user mask
		    userMask.n_members++;
		    //enables or disables components in the window
		    default_button.setEnabled(true);
		    removeList.setEnabled(true);
		    from_string.setText("");
		    to_string.setText("");
		    // adds the new set in the usermask
		    userMask.setData(s1,s2,pos);
		    userMask.sort();
		    //update the view of the table
		    UpdateTable(userMask.getData());
		    fireChange();
		}
		else { 
		    JOptionPane.showMessageDialog(mainframe,"There is no more space; Please remove one entry from the table","",JOptionPane.INFORMATION_MESSAGE);
		}
	    }
	}
	else if (b== default_button) {
	    //retrieve the default mask
	    userMask = new CharactersToBeReplaced(new DefaultCharactersToBeReplaced());
	    //update the view of the table
	    UpdateTable(userMask.getData());
	    fireChange();
	}
	else if(b==removeList) {
	    //if there is a row selected in the table
	    if ((pos = getSelection(table)) != -1) {
		//if this selected row contains data
		if (pos < userMask.n_members) {
		    //makes the changes
		    userMask.setData(pos);
		    userMask.n_members--;
		    //update the view of the table
		    UpdateTable(userMask.getData());
		    fireChange();
		}
	    }
	}
	// saves the user mask on the disk
	else if(b==write_default) userMask.writeData();
	//dismiss the window
	else if(b==dismiss) HideDialog();
    }

    /**
     * updates the view of the table
     *@param s the mask with which the table is updated
     */
    private void UpdateTable(String[][]  s){
	for(int i=0;i<s.length;i++){
	    myModel.setValueAt(s[i][0],i,0);
	    myModel.setValueAt(s[i][1],i,1);
	}
    }

    /**
     * @param table the table where the selection is done
     * @return the number of the selected row, if it contains data or -1 otherwise
     */
    private int getSelection(JTable table){
	for (int j = 0; j < REPLACEMASKSIZE; j++)
	    if (table.isRowSelected(j))
		return j;

	return -1;
    }


    /**
     *when a change occurs stateChanged is alled on the registered classes
     * @see UserInterface#changed()
     */
    protected void fireChange() {
	ui.changed();
    }
}


/**
 * this class implements a default set of strings to be replaced
 */
class DefaultCharactersToBeReplaced implements UserOptions { 
    int n_members = 6;
    final String DefaultsFileName = new String(DEFAULTSFILENAME);
    /**
     * used when the file containing the data does not exist
     */
    String[][] data= new String[][] {{"\\\"a","ae"},{"\\\"o","oe"},{"\"a","ae"},{"\"o","oe"},{"\\\"u","ue"},{"\"u","ue"},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "},{" "," "}}; 

    /**
     * builts the matrix by reading it from the disk
     * it is stored in a file useroptions.txt
     * if the file does not exist the matrix #data is used
     */
    public DefaultCharactersToBeReplaced() {
	try {
	    //reads the file
	    File defaults_file = new File(DefaultsFileName);
	    if (defaults_file.exists()) {
		data = new String[REPLACEMASKSIZE][2];
		FileReader in = new FileReader(defaults_file);
		StreamTokenizer tok=new StreamTokenizer(in);
		int i=0;
		tok.resetSyntax();
		tok.whitespaceChars(60,60);
		tok.whitespaceChars(62,62);
		tok.wordChars(32,59);
		tok.wordChars(61,61);
		tok.wordChars(63,127);
		tok.eolIsSignificant(true);
		try{
		    while(tok.nextToken()!=tok.TT_EOF){
			if (tok.ttype == tok.TT_WORD) {
			    int j=0;
			    String s=tok.sval;
			    //separates the two parts of one line (the replaced character and the one to replace with
			    StringTokenizer tok_string=new StringTokenizer(s,";");
			    while(tok_string.hasMoreTokens()){
				data[i][j]=tok_string.nextToken();
			  
				j++;
			    }
			    n_members=i+1;
			}
			else if(tok.ttype == tok.TT_EOL)
			    i++;
		    }
		    in.close();
		}
		catch (IOException e){}
	    }
	}
	catch(IOException e){}
    }
	
    /**
     * @return the default mask
     */
    public String[][] getData() {
	return data;
    }

    /**
     * @return the number of rows containing data
     */
    public int getMembersNumber() {
	return n_members;
    }
    
    /**
     * @param i the number of the row
     * @return the ith row of data
     */
    public String[] getMemberAt(int i) {
	return data[i];
    }

}


/**
 * this class builts the user mask
 */
class CharactersToBeReplaced implements UserOptions{
    String[][] data;
    int n_members;

    /**
     * takes the default mask as the user's mask and orders it 
     * @see #sort()
     */
    public CharactersToBeReplaced(DefaultCharactersToBeReplaced def) {
	n_members = def.getMembersNumber();	
	data = new String[REPLACEMASKSIZE][2];
	for(int i=0; i < n_members; i++) {
	    data[i] = def.getMemberAt(i);
	}
	sort();
    }

    /**
     * @return the user's mask
     */
    public String[][] getData() {
	return this.data;
    }  


    /**
     * implements the bubble sort algorithm
     * the sorting is done according to the length of the characters to be replaced.
     * the sorting is done in decreasing order
     */
    public void sort(){
	boolean found = true;
	String[] aux;
	if (n_members > 1)
	    while(found){
		found=false;
		for(int i=0;i<(n_members)-1;i++){
		    if(data[i][0].length()<data[i+1][0].length()){
			aux=data[i];
			data[i]=data[i+1];
			data[i+1]=aux;
			found=true;
		    }
		}
	    }
    }


    /**
     * inserts a new line in the matrix
     * @param os the string to be replaced
     * @param ns the string to replace with
     * @paaram i he position where to insert the row
     */
    public void setData(String os, String ns, int pos){
	this.data[pos][0]=os;
	this.data[pos][1]=ns;
    }

    /**
     * rearanges the data when a row is deleted
     * @param pos the number of row which is deleted
     */
    public void setData(int pos) {
	int i;
	for (i=pos;i<n_members;i++){
	    data[i][0] = data[i+1][0];
	    data[i][1] = data[i+1][1];
	}
    }
    

    /**
     * writes the user's mask to the disk
     */
    public void writeData() {
	String out_filename = new String(DEFAULTSFILENAME);
	try {
	    DataOutputStream out = new DataOutputStream(new FileOutputStream(out_filename));
	    for(int i=0;i<n_members;i++){
		out.writeBytes(data[i][0]+";"+data[i][1]+"\n");
	    }
	    out.close();
	}
	catch(IOException e){}
    }

    /**
     * @return the number of rows containing data in the user's mask
     */
    public int getMembersNumber() {
	return n_members;
    }

}


