import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.StringTokenizer;
import javax.swing.table.*;

/**
 * This class computes statstics concerning the performances of the 5 types of translation
 * @see UserOptions
 */
class TranslationStatistics implements Runnable,UserOptions {

    JDialog dialog = null;
    boolean isShown = false;

    String best;
    String chosen;
    String trans[] = new String[4];
    int best_trans[][] = new int[4][TRANSLATION_SEGMENTS];  // no more than TRANSLATION_SEGMENTS segments for translation allowed

    float segments[] = new float[4];
    float turns[] = new float[4];
    float chosen_segments[] = new float[4];	
    float chosen_turns[] = new float[4];	
    float best_chosen_segments[] = new float[4];
    float best_chosen_turns[] = new float[4];
   
    JFrame mainframe;
    UserInterface ui;
    String command;
    DefaultTableModel myModel;

    final String[] columnNames={"Translation ","Chosen Segm. %"};
    final String[][] data={
	{"Case-Based "," "},
	{"Statistical "," "},
	{"Dialog Based "," "},
	{"Deep Analysis"," "},
    };
    final String[] columns_aux={"Chosen Segm.","Chosen Turns %","Chosen Turns","Best Chosen Segm. %","Best Chosen Segm.", "Best Chosen Turns %","Best Chosen Turns"};
    final String[] data_aux={" "," ", " ", " ", " ", " ", " "};

    /**
     * builts the window and the widgets
     */
    TranslationStatistics(JFrame mainframe, UserInterface ui) {
	this.mainframe = mainframe;
	this.ui = ui;
	Color light = new Color(102, 255, 255);
	Color dark = new Color(051, 204, 255);
	dialog = new JDialog(mainframe, "Translation Statistics");
	dialog.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    isShown = false;
		    dialog.hide();
		}
	    });

	//the table model
	myModel=new DefaultTableModel(data,columnNames);
	for(int i=0;i<columns_aux.length;i++){
	    myModel.addColumn(columns_aux[i],data_aux);
	}
	JTable table=new JTable(myModel);
	table.setPreferredScrollableViewportSize(new Dimension(1200,65));
	Container c = dialog.getContentPane();
	c.setLayout(new BorderLayout());
	JScrollPane scrollPane=new JScrollPane(table);
	scrollPane.createHorizontalScrollBar();

	c.add(scrollPane, BorderLayout.CENTER);
	JPanel sud = new JPanel(new FlowLayout());
	JButton update = new JButton("update");
	JButton dismiss = new JButton("Close");
	update.addActionListener(new ActionListener () {
		public void actionPerformed(ActionEvent e) {
		    command=e.getActionCommand();
		    final SwingWorker worker= new SwingWorker(){
			    public Object construct(){
				Update();
				return command;
			    }
			};
		    worker.start();
		}
	    });
	dismiss.addActionListener(new ActionListener () {
		public void actionPerformed(ActionEvent e) {
		    HideDialog();
		}
	    });
	sud.add(update);
	sud.add(dismiss);
	c.add(sud, BorderLayout.SOUTH);
	dialog.pack();
    }


    /**
     * dismiss the window
     */
    public void HideDialog() {
	isShown = false;
	dialog.hide();
    }


    /**
     * pops-up the window and starts a thread for computations
     * @see #UpdateDialog()
     */
    public void ShowDialog() {
	isShown = true;
	dialog.show();
	Thread t = new Thread(this);
	t.start();
	try{
	    t.join();
	}
	catch(InterruptedException e){}
	UpdateDialog();
    }


    /**
     * called when a change is done by the user on some criteria or constraints in the user interface
     * @see #UpdateDialog()
     */    
    public void Update() {
	isShown=true;
	Thread t = new Thread(this);
	t.start();
	try{
	    t.join();}
	catch(InterruptedException e){}
	UpdateDialog();

    }

    /**
     * starts the computations
     * @see #Calculate()
     */
    public void run() {
	Calculate();
    }

    /**
     * updates the view of the table
     */
    private void UpdateDialog() {
	String data;
	if(isShown){
	    for (int i = 0; i< 4; i++) {
		data=(String)myModel.getValueAt(i,1);
		data=""+(chosen_segments[i]/segments[i])*100+"%";
		myModel.setValueAt(data,i,1);
		data=(String)myModel.getValueAt(i,2);
		data="("+chosen_segments[i]+"/"+segments[i]+")";
		myModel.setValueAt(data,i,2);
		data=(String)myModel.getValueAt(i,3);
		data=""+(chosen_turns[i]/turns[i])*100+"%";
		myModel.setValueAt(data,i,3);
		data=(String)myModel.getValueAt(i,4);
		data="("+chosen_turns[i]+"/"+turns[i]+")";
		myModel.setValueAt(data,i,4);
		data=(String)myModel.getValueAt(i,5);
		data=""+(best_chosen_turns[i]/turns[i])*100+"%";   
		myModel.setValueAt(data,i,5);
		data=(String)myModel.getValueAt(i,6);
		data="("+best_chosen_turns[i]+"/"+turns[i]+")";
		myModel.setValueAt(data,i,6);
		data=(String)myModel.getValueAt(i,7);
		data=""+(best_chosen_segments[i]/segments[i])*100+"%";
		myModel.setValueAt(data,i,7);
		data=(String)myModel.getValueAt(i,8);
		data="("+best_chosen_segments[i]+"/"+segments[i]+")";
		myModel.setValueAt(data,i,8);
	    }
	}
    }

    /**
     * initialize the data
     */
    private void init() {
	for (int i = 0; i< 4; i++) {
	    segments[i] = 0;
	    turns[i] = 0;
	    chosen_segments[i] = 0;
	    chosen_turns[i] = 0;
	    best_chosen_segments[i] = 0;
	    best_chosen_turns[i] = 0;
	}
    }

    /**
     * retrieve the data
     * @param j the current turn
     */

    private void GetTurn(Judgements j) {
	best = (String) j.turn.get("BEST");
	chosen = (String) j.turn.get("OUT");
	trans[0] = (String) j.turn.get("ALI");
	trans[1] = (String) j.turn.get("STAT");
	trans[2] = (String) j.turn.get("SYN");
	trans[3] = (String) j.turn.get("DEEP");
    }


    /**
     * computes the statistics
     * @see #ParseChosen(int)
     * @see #ParseBest()
     */
    private void Calculate() {
	String value;
	Judgements j = ui.getCurrentDocument().getFirst();
	// initialize variables
	init();
	int turn_number = ui.getCurrentDocument().turn_number;
	// compute turn by turn
	for (int i = 0; i<turn_number; i++) {
	    GetTurn(j);
	    ParseBest();
	    for (int k = 0; k < 4; k++) { // call Parse once for every translation module
		ParseChosen(k);
	    }
	    if (i < turn_number -1 )
		j = (Judgements) j.turn.get("NEXT");	    
	}
    }



    private void ParseBest() {
	for (int i = 0; i < 4; i++)
	    for (int j = 0; j < TRANSLATION_SEGMENTS; j++)
		best_trans[i][j] = 0;
	int module = -1;
	if (best != null) {
	    StringTokenizer module_token = new StringTokenizer(best, "|");
	    while (module_token.hasMoreTokens()) {
		module++;
		StringTokenizer best_token = new StringTokenizer(module_token.nextToken(), ":");
		if (best_token.countTokens() == 2) {
		    best_token.nextToken();
		    StringTokenizer number_token = new StringTokenizer(best_token.nextToken(), ",");
		    while (number_token.hasMoreTokens()) {
			best_trans[module][Integer.parseInt(number_token.nextToken())] = 1;
		    }
		}
	    }
	}
    }


    private void ParseChosen(int k) {
	boolean single_token;
	String text;
	String translation;
	int segment_number;
	if ((chosen != null)&&(trans[k] != null)) {
	    StringTokenizer text_tok = new StringTokenizer(chosen, "|");
	    StringTokenizer trans_tok = new StringTokenizer(trans[k], "|");
	    // update the number of segments (or turns) in the translation
	    single_token = false;
	    if (trans_tok.countTokens() == 1) {
		single_token = true;
		turns[k]++;
	    }
	    else if (trans_tok.countTokens() > 1)
		segments[k] += trans_tok.countTokens();
	    // look for chosed segements or turns
	    while (text_tok.hasMoreTokens()) {
		// retrieve the segment of the chosen translation
		text = text_tok.nextToken();
		text = DeleteDotsAndCommas(text);
		text = text.trim();
		// parse the k-th translation module
		trans_tok = new StringTokenizer(trans[k], "|"); // have to do it again
		segment_number = 0;
		while (trans_tok.hasMoreTokens()) {
		    // retrieve the segment from the translation
		    translation = trans_tok.nextToken();
		    segment_number++;
		    translation = DeleteDotsAndCommas(translation);
		    translation = translation.trim();
		    if (text.equalsIgnoreCase(translation)) {
			// if there is only one segment, update the whole sentece counter
			// otherwise update the segment counter 
			if (single_token)
			    chosen_turns[k]++;
			else
			    chosen_segments[k]++;
			// now let's see wheter the translation was a best one
			if (best_trans[k][segment_number] == 1) { 
			    if (single_token)
				best_chosen_turns[k]++;
			    else
				best_chosen_segments[k]++;			    
			}
		    }
		}
	    }
	}
    }
   

    private boolean isSelected(String segment, int number) {
	int token_number = 0;
	String text = new String(chosen);
	StringTokenizer tok = new StringTokenizer(text, "|");
	while (tok.hasMoreTokens()) {
	    text = tok.nextToken();
	    token_number++;
	    text = chosen.trim();
	    segment = segment.trim();
	    text = DeleteDotsAndCommas(text);
	    segment = DeleteDotsAndCommas(segment);
	    if (number == token_number)
		if (text.equalsIgnoreCase(segment))
		    return true;

	}
	return false;
    }


    private boolean isDotOrComma(char a_char) {
	if (a_char == '.')
	    return true;
	if (a_char == ',')
	    return true;
	if (a_char == ',')
	    return true;
	if (a_char == ';')
	    return true;
	if (a_char == ':')
	    return true;
	if (a_char == '\"')
	    return true;
	if (a_char == '\'')
	    return true;
	if (a_char == '!')
	    return true;
	if (a_char == '?')
	    return true;
	return false;
    }


    private String DeleteDotsAndCommas(String a_string) {
	String another_string = new String("");
	for (int i = 0; i< a_string.length(); i++) {
	    if (!isDotOrComma(a_string.charAt(i)))
		another_string += a_string.charAt(i);
	}	
	return another_string;    
    }
 
}
