import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JRadioButton;
import java.util.Hashtable;

/**
 *Shows the current turn to be evaluated and the general quality tags related to the translation.
 * The class extends the JPanel class in order to be directly nested in the layout of the 
 * application and implemets the ActionListener interface to listen to the events on the widgets.
*/
public class CurrentTurnPanel extends JPanel implements ActionListener {
    // all the widgets of this panel
    JTextArea CurrentInput;
    JTextArea CurrentOutput;
    JRadioButton TranslationMismatchYes;
    JRadioButton TranslationMismatchNo;
    JRadioButton TranslationHuman;
    JRadioButton TranslationMachine;
    JRadioButton TranslationQualityGood;
    JRadioButton TranslationQualityIntermediate;
    JRadioButton TranslationQualityBad;
    // button groups have to be declared here to be able to 
    // set 'no selection' on their widgets
    ButtonGroup soundgroup;
    ButtonGroup mismatchgroup;
    ButtonGroup qualitygroup;
    // a reference to the user interface which is set 
    // by the constructor of this class
    UserInterface ui;    
    // outputs contains the string that identifies the translation 
    // to be shown in the output text area of this panel
    String outputs[] = {"OUT", "ALI", "STAT", "SYN", "DEEP", "PRE"};

   
    /**
     * This method access the user interface of the application
     * @see UserInterface
     */
    public UserInterface getUI() {
	return ui;
    }

    /**
     * The constructor of the class takes as parameter the 
     * referrence to the user interface of the application
     * @param ui reference to the user interface of the application
     */
    public CurrentTurnPanel(UserInterface ui) {
	// creates the main panel and sets its layout
	this.ui=ui;
	this.setLayout(new BorderLayout());
	// creates a border for the panel
	this.setBorder(BorderFactory.createEmptyBorder(5,20,0,20));
	// creates and add the text pane for the input sentence
	CurrentInput = new JTextArea(4, 20);
	CurrentInput.setLineWrap(true);
	CurrentInput.setWrapStyleWord(true);
	CurrentInput.setEditable(false);
	this.add(new JScrollPane(CurrentInput,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER ), BorderLayout.WEST);

	// creates a subpanel for translation mismatch
	JPanel p = new JPanel(new BorderLayout());
	// translation mismatch
	JLabel l = new JLabel("Translation Mismatch", JLabel.CENTER);
	p.add(l, BorderLayout.NORTH);
	// instantiates the radio button for translation mismatch
	TranslationMismatchYes = new JRadioButton("Yes");
        TranslationMismatchYes.setActionCommand("TRANS_MISM_YES");
	TranslationMismatchNo = new JRadioButton("No");
        TranslationMismatchNo.setActionCommand("TRANS_MISM_NO");
	// Group the radio buttons for translation mismatch.
        mismatchgroup = new ButtonGroup();
	mismatchgroup.add(TranslationMismatchNo);
	mismatchgroup.add(TranslationMismatchYes);
        JPanel radioPanelMismatch = new JPanel();
        radioPanelMismatch.setLayout(new FlowLayout());
        radioPanelMismatch.add(TranslationMismatchYes, 
			       (int) JPanel.CENTER_ALIGNMENT);
        radioPanelMismatch.add(TranslationMismatchNo, 
			       (int) JPanel.CENTER_ALIGNMENT);	
	p.add(radioPanelMismatch, BorderLayout.SOUTH);
        p.setBorder(BorderFactory.createEtchedBorder());
	// cretes a subpanel for translation soundiness
	JPanel r = new JPanel(new BorderLayout());
	// translation soundiness
	l = new JLabel("Translation Soundness", JLabel.CENTER);
	r.add(l, BorderLayout.NORTH);
	// instantiates the radio button for translation soundiness
	TranslationHuman = new JRadioButton("Human");
        TranslationHuman.setActionCommand("TRANS_HUMAN");
	TranslationMachine = new JRadioButton("Machine");
        TranslationMachine.setActionCommand("TRANS_MACHINE");
	// Group the radio buttons for translation soundiness
        soundgroup = new ButtonGroup();
	soundgroup.add(TranslationMachine);
	soundgroup.add(TranslationHuman);
        JPanel radioPanelSound = new JPanel();
        radioPanelSound.setLayout(new FlowLayout());
        radioPanelSound.add(TranslationHuman, (int) JPanel.CENTER_ALIGNMENT);
        radioPanelSound.add(TranslationMachine, (int) JPanel.CENTER_ALIGNMENT);	
	r.add(radioPanelSound, BorderLayout.SOUTH);
        r.setBorder(BorderFactory.createEtchedBorder());
	// cretes a subpanel for translation quality
	JPanel q = new JPanel(new BorderLayout());
	// translation quality
	JLabel l1 = new JLabel("Translation Quality", JLabel.CENTER);
	q.add(l1, BorderLayout.NORTH);
	// instantiates the radio button for translation wuslity
	TranslationQualityGood = new JRadioButton("Good");
        TranslationQualityGood.setActionCommand("TRANS_QUAL_GOOD");
	TranslationQualityIntermediate = new JRadioButton("Intermediate");
        TranslationQualityIntermediate.setActionCommand("TRANS_QUAL_INTERM");
	TranslationQualityBad = new JRadioButton("Bad");
        TranslationQualityBad.setActionCommand("TRANS_QUAL_BAD");
	// Group the radio buttons for translation quality.
	qualitygroup = new ButtonGroup();
	qualitygroup.add(TranslationQualityGood);
	qualitygroup.add(TranslationQualityIntermediate);
	qualitygroup.add(TranslationQualityBad);
        JPanel radioPanelQuality = new JPanel();
        radioPanelQuality.setLayout(new FlowLayout());
        radioPanelQuality.add(TranslationQualityGood);
        radioPanelQuality.add(TranslationQualityIntermediate);	
        radioPanelQuality.add(TranslationQualityBad);	
	q.add(radioPanelQuality, BorderLayout.SOUTH);
        q.setBorder(BorderFactory.createEtchedBorder());
	// adds panels of radiobuttons to the mainpanel
	JPanel mainpanelCenter = new JPanel(new GridLayout(3,1,0,10));
	mainpanelCenter.add(p);
	mainpanelCenter.add(r);
	mainpanelCenter.add(q);
	mainpanelCenter.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
	this.add(mainpanelCenter, BorderLayout.CENTER);
	// creates and add the text pane for the output sentence
	CurrentOutput = new JTextArea(4, 20);
	CurrentOutput.setLineWrap(true);
	CurrentOutput.setWrapStyleWord(true);
	CurrentOutput.setEditable(false);
	this.add(new JScrollPane(CurrentOutput,JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,JScrollPane.HORIZONTAL_SCROLLBAR_NEVER ), BorderLayout.EAST);
	// adds all listeners
	TranslationMismatchYes.addActionListener(this);
	TranslationMismatchNo.addActionListener(this);
	TranslationQualityGood.addActionListener(this); 
	TranslationQualityIntermediate.addActionListener(this);
	TranslationQualityBad.addActionListener(this);
	TranslationHuman.addActionListener(this);
	TranslationMachine.addActionListener(this);
    }


    /** 
     *this method gets the Source of the action and executes
     *the related action (update the document)
     */	
    public void actionPerformed(ActionEvent e) {	
	if(e.getSource() == TranslationMismatchYes) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("EFF", "1");
	else if (e.getSource() == TranslationMismatchNo) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("EFF", "0");
	else if (e.getSource() == TranslationQualityGood) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("QUAL", "1");
	else if (e.getSource() == TranslationQualityIntermediate) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("QUAL", "3");
	else if (e.getSource() == TranslationQualityBad) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("QUAL", "0");
	else if (e.getSource() == TranslationHuman) 
	    ui.getCurrentDocument().CurrentTurn.turn.put("SOUND", "H");
	else if (e.getSource() == TranslationMachine)
	    ui.getCurrentDocument().CurrentTurn.turn.put("SOUND", "M");
	// activate the save manu
	ui.menu.setSavable(true);
    }   

   
   
    /**
     * This method updates the panel when the turn to be shown changes
     * @param j is the current turn to be shown
     */
    public void changed(Judgements j) {
	// this variable contains the value of a parameter in the current turn
	String state;
	// the input sentence is updated
	if(j!=null){
	    if (j.turn.get("IN") != null) 
		CurrentInput.setText(ui.Options.substitute((String) j.turn.get("IN")));
	    else
		CurrentInput.setText("");
	    // the output sentecne is updated
	    if (j.turn.get(outputs[ui.getCurrentDocument().output-1]) != null) 
		CurrentOutput.setText(ui.Options.substitute((String) j.turn.get(outputs[ui.getCurrentDocument().output-1])));
	    else 
		CurrentOutput.setText("");
	    // the translation mismatch parameter is retrieved...	
	    state = (String) j.turn.get("EFF");
	    // ... and the corresponding widget is updated
	    // EFF = 1 -> Translation Mismatch
	    // EFF = 0 -> No Translation Mismatch
	    // EFF = 2 -> No Selection
	    if (state != null) {
		if (state.equals("1")) {
		    TranslationMismatchYes.setSelected(true);
		    TranslationMismatchNo.setSelected(false);
		}
		else if (state.equals("0")){
		    TranslationMismatchYes.setSelected(false);
		    TranslationMismatchNo.setSelected(true);
		}
		else {
		    mismatchgroup.setSelected(null, true);
		}
	    }
	    else {
		mismatchgroup.setSelected(null, true);
	    }
	    // the translation quality parameter is retrieved...	
	    state = (String) j.turn.get("QUAL");
	    // ... and the corresponding widget is updated
	    // QUAL = 1 -> Good
	    // QUAL = 0 -> Bad
	    // QUAL = 2 -> No Selection
	    // QUAL = 3 -> Intermediate
	    if (state != null) {
		if (state.equals("0")) {
		    TranslationQualityBad.setSelected(true);
		    TranslationQualityGood.setSelected(false);
		    TranslationQualityIntermediate.setSelected(false);
		}
		else if(state.equals("1")) {
		    TranslationQualityBad.setSelected(false);
		    TranslationQualityGood.setSelected(true);
		    TranslationQualityIntermediate.setSelected(false);
		}
		else if(state.equals("3")) {
		    TranslationQualityBad.setSelected(false);
		    TranslationQualityGood.setSelected(false);
		    TranslationQualityIntermediate.setSelected(true);
		}
		else {
		    qualitygroup.setSelected(null, true);
		}
	    }
	    else {
		qualitygroup.setSelected(null, true);
	    }
	    // the translation soundiness parameter is retrieved...
	    state = (String) j.turn.get("SOUND");
	    // ... and the corresponding widget is updated
	    // SOUND = 1 -> Good
	    // SOUND = 0 -> Bad
	    // QUAL = 2 -> No Selection
	    // QUAL = 3 -> Intermediate
	    if (state != null) {
		if (state.equals("H")) {
		    TranslationHuman.setSelected(true);
		}
		else if(state.equals("M")) {
		    TranslationMachine.setSelected(true);
		}
		else {
		    soundgroup.setSelected(null, true);
		}
	    }
	    else {
		soundgroup.setSelected(null, true);
	    }
	}
    }

}
