import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.StringTokenizer;
/**
 * This class builts the DialogActPanel, a window (instance of JDialog) which shows for 
 * each input or output turn it's role in the dialogue: great, bye, politness formula etc. 
 * It implements the ActionListener interface for controlling the buttons of the window. 
 * There are three buttons, one for the input term, one for the output term and one for 
 * closing the window
 * if the option "Edit data" is selected in the main menu the user can sets and change 
 * the type of dialogue act for each turn 
 */
public class DialogActPanel implements ActionListener{
    JDialog dialog = null;
    JPanel nord;
    JTextArea turn;
    // CONTROL DIALOGUE
    JToggleButton greet;
    JToggleButton bye;
    JToggleButton introduce;
    JToggleButton politeness_formula;
    JToggleButton thank;
    JToggleButton deliberate;
    JToggleButton backchannel;
    // MANAGE TASK
    JToggleButton init;
    JToggleButton defer;
    JToggleButton close;
    // PROMOTE TASK
    JToggleButton request_suggest;
    JToggleButton request_clarify;
    JToggleButton request_comment;
    JToggleButton request_commit;
    JToggleButton suggest;
    JToggleButton inform_digress;
    JToggleButton inform_exclude;
    JToggleButton inform_clarify;
    JToggleButton inform_give_reason;
    JToggleButton accept;
    JToggleButton confirm;
    JToggleButton reject;
    JToggleButton commit;
    JToggleButton offer;
    // source
    JToggleButton input;
    JToggleButton output;
    JButton button1;
    JFrame mainframe;
    UserInterface ui;
    Document doc;    
    /**
     * @return the panel situated at the north area of the layout
     */
    public JPanel getDialogActPanelNord() {
	return nord;
    }
    /**
     * packs all components and shows the Dialog Act window
     */
    public void ShowDialog() {
	dialog.pack();
	init();
	dialog.show();
    }

    /**
     * @return the current opened Document
     * @see UserInterface#getCurrentDocument()
     */
    public Document getDoc() {
	return ui.getCurrentDocument();
    }   
    /**
     * @return the user interface instance with the current settings
     * @see UserInterface
     */
    public UserInterface getUI(){
	return ui;
    }
    /**
     * shows in the text area the first turn of the dialogue with the corresponding 
     * properties and sets the button "Input" to true 
     * @see Document#getData()
     * @see #changed(Judgements)
     */
    private void init() {
	Judgements j = ui.getCurrentDocument().getData();
	input.setSelected(true);
	if (j != null)
	    changed(j);
    }
    /**
     * Processes the events occured when one of the buttons Input or Output is 
     * selected and enables the Save option in the main menu
     * @see MyMenu#setSavable(boolean)
     */
    public void actionPerformed(ActionEvent e) {
	if (input.isSelected()) {	    
	    ui.getCurrentDocument().CurrentTurn.turn.put("DIALIN", Check());
	}
	else {
	    ui.getCurrentDocument().CurrentTurn.turn.put("DIALOUT", Check());
	}
	ui.menu.setSavable(true);

    }

    /**
     * The constructor of the DialogActPanel receives a reference of the mainframe, 
     * UserInterface and the current document. We pass all these three arguments 
     * instead of only mainframe to avoid successively calls as getUserInterface(), getCurrentDocument()
     * @see MainFrame
     * @see UserInterface 
     * @see Document
     */
    public DialogActPanel(JFrame mainframe, UserInterface ui,Document doc) {
	// set the characteristics of text areas
	turn = new JTextArea(4, 20);
	turn.setLineWrap(true);
	turn.setWrapStyleWord(true);
	turn.setEditable(false);
	this.mainframe=mainframe; 
	this.ui=ui;
	this.doc=doc;
	dialog = new JDialog(this.mainframe, "Dialog act");
	Container c = dialog.getContentPane();
	c.setLayout(new BorderLayout());

	nord = new JPanel(new BorderLayout());
	JPanel nordwest = new JPanel(new GridLayout(12,0));
	// CONTROL DIALOGUE
	Color bgcolor = new Color(102, 255, 255);
	greet = new JToggleButton("greet");
	greet.addActionListener(this);
	greet.setBackground(bgcolor);
	bye = new JToggleButton("bye");
	bye.addActionListener(this);
	bye.setBackground(bgcolor);
	introduce = new JToggleButton("introduce");
	introduce.addActionListener(this);
	introduce.setBackground(bgcolor);
	politeness_formula = new JToggleButton("politeness formula");
	politeness_formula.addActionListener(this);
	politeness_formula.setBackground(bgcolor);
	thank = new JToggleButton("thank");
	thank.addActionListener(this);
	thank.setBackground(bgcolor);
	deliberate = new JToggleButton("deliberate");
	deliberate.addActionListener(this);
	deliberate.setBackground(bgcolor);
	backchannel = new JToggleButton("backchannel");
	backchannel.addActionListener(this);
	backchannel.setBackground(bgcolor);
	// MANAGE TASK
	bgcolor = new Color(051, 102, 255);
	init = new JToggleButton("init");
	init.addActionListener(this);
	init.setBackground(bgcolor);
	defer = new JToggleButton("defer");
	defer.addActionListener(this);
	defer.setBackground(bgcolor);
	close = new JToggleButton("close");
	close.addActionListener(this);
	close.setBackground(bgcolor);
	bgcolor = new Color(051, 204, 255);
	commit = new JToggleButton("commit");
	commit.addActionListener(this);
	commit.setBackground(bgcolor);
	offer = new JToggleButton("offer");
	offer.addActionListener(this);
	offer.setBackground(bgcolor);
	

	nordwest.add(greet);
	nordwest.add(bye);
	nordwest.add(introduce);
	nordwest.add(politeness_formula);
	nordwest.add(thank);
	nordwest.add(deliberate);
	nordwest.add(backchannel);
	nordwest.add(init);
	nordwest.add(defer);
	nordwest.add(close);
	nordwest.add(commit);
	nordwest.add(offer);

	nord.add(nordwest, BorderLayout.WEST);

	nord.add(new JScrollPane(turn), BorderLayout.CENTER);

	request_suggest = new JToggleButton("request suggest");
	request_suggest.addActionListener(this);
	request_suggest.setBackground(bgcolor);
	request_clarify = new JToggleButton("request clarify");
	request_clarify.addActionListener(this);
	request_clarify.setBackground(bgcolor);
	request_comment =  new JToggleButton("request comment");
	request_comment.addActionListener(this);
	request_comment.setBackground(bgcolor);
	request_commit = new JToggleButton("request commit");
	request_commit.addActionListener(this);
	request_commit.setBackground(bgcolor);

	suggest = new JToggleButton("suggest");
	suggest.addActionListener(this);
	suggest.setBackground(bgcolor);

	inform_digress = new JToggleButton("inform digress");
	inform_digress.addActionListener(this);
	inform_digress.setBackground(bgcolor);
	inform_exclude = new JToggleButton("inform exclude");
	inform_exclude.addActionListener(this);
	inform_exclude.setBackground(bgcolor);
	inform_clarify = new JToggleButton("inform clarify ");
	inform_clarify.addActionListener(this);
	inform_clarify .setBackground(bgcolor);
	inform_give_reason = new JToggleButton("inform give reason");
	inform_give_reason.addActionListener(this);
	inform_give_reason.setBackground(bgcolor);
	accept = new JToggleButton("accept");
	accept.addActionListener(this);
	accept.setBackground(bgcolor);
	confirm = new JToggleButton("confirm");
	confirm.addActionListener(this);
	confirm.setBackground(bgcolor);
	reject = new JToggleButton("reject");
	reject.addActionListener(this);
	reject.setBackground(bgcolor);

	JPanel nordeast = new JPanel(new GridLayout(12,0));

	nordeast.add(request_suggest);
	nordeast.add(request_clarify);
	nordeast.add(request_comment);
	nordeast.add(request_commit);
	nordeast.add(suggest);
	nordeast.add(inform_digress);
	nordeast.add(inform_exclude);
	nordeast.add(inform_clarify);
	nordeast.add(inform_give_reason);
	nordeast.add(accept);
	nordeast.add(confirm);
	nordeast.add(reject);

	nord.add(nordeast, BorderLayout.EAST);

	c.add(nord, BorderLayout.NORTH);

	

	JPanel sud = new JPanel(new FlowLayout());
	ButtonGroup bg = new ButtonGroup();
	input = new JToggleButton("Input turn");
	bg.add(input);	
	input.addActionListener(new ActionListener () {
		public void actionPerformed(ActionEvent e) {
		    input.setSelected(true);
		    output.setSelected(false);
		    Judgements j = getDoc().getData();
		    if (j != null)
			changed(j);
		}
	    });
	output = new JToggleButton("Output turn");
	bg.add(output);
	output.addActionListener(new ActionListener () {
		public void actionPerformed(ActionEvent e) {
		    input.setSelected(false);
		    output.setSelected(true);
		    Judgements j = getDoc().getData();
		    if (j != null)
			changed(j);
		    
		}
	    });

	JButton button1 = new JButton("Close");
	button1.addActionListener(new ActionListener () {
		public void actionPerformed(ActionEvent e) {
		    dialog.hide();
		}
	    });

	sud.add(input);
	sud.add(output);
	sud.add(button1);
	c.add(sud, BorderLayout.SOUTH);
    }
    /**
     * @return the current window
     */
    public JDialog getTranslationDialog() {
	return dialog;
    }
    
    /**
     * checks which options (JToggleButtons) are selected and depending on this sets 
     * the respective buttons as selected
     * called by {@link #changed(Judgements) changed}
     */
    private void Check(String value) {
	if (value.equals("greet"))
	    greet.setSelected(true);
	if (value.equals("bye"))
	    bye.setSelected(true);
	if (value.equals("introduce"))
	    introduce.setSelected(true);
	if (value.equals("politeness_formula"))
	    politeness_formula.setSelected(true);
	if (value.equals("thank"))
	    thank.setSelected(true);
	if (value.equals("deliberate"))
	    deliberate.setSelected(true);
	if (value.equals("backchannel"))
	    backchannel.setSelected(true);
	if (value.equals("init"))
	    init.setSelected(true);
	if (value.equals("defer"))
	    defer.setSelected(true);	
	if (value.equals("close"))
	    close.setSelected(true);	
	if (value.equals("request_suggest"))
	    request_suggest.setSelected(true);
	if (value.equals("request_clarify"))
	    request_clarify.setSelected(true);
	if (value.equals("request_comment"))
	    request_comment.setSelected(true);	
	if (value.equals("request_commit"))
	    request_commit.setSelected(true);
	if (value.equals("suggest"))
	    suggest.setSelected(true);	
	if (value.equals("inform_digress"))
	    inform_digress.setSelected(true);
	if (value.equals("inform_exclude"))
	    inform_exclude.setSelected(true);
	if (value.equals("inform_clarify"))
	    inform_clarify.setSelected(true);	
	if (value.equals("inform_give_reason"))
	    inform_give_reason.setSelected(true);
	if (value.equals("accept"))
	    accept.setSelected(true);
	if (value.equals("confirm"))
	    confirm.setSelected(true);
	if (value.equals("reject"))
	    reject.setSelected(true);
	if (value.equals("commit"))
	    commit .setSelected(true);	
	if (value.equals("offer"))
	    offer.setSelected(true);
    }
    /**
     * sets different buttons to the value of the boolean parameter
     * called by {@link #changed(Judgements j) changed}
     */
    private void Check(boolean check) {
	greet.setSelected(check);
	bye.setSelected(check);
	introduce.setSelected(check);
	politeness_formula.setSelected(check);
	thank.setSelected(check);
	deliberate.setSelected(check);
	backchannel.setSelected(check);
	init.setSelected(check);
	defer.setSelected(check);
	close.setSelected(check);	
	request_suggest.setSelected(check);
	request_clarify.setSelected(check);
	request_comment.setSelected(check);	
	request_commit.setSelected(check);
	suggest.setSelected(check);	
	inform_digress.setSelected(check);
	inform_exclude.setSelected(check);
	inform_clarify.setSelected(check);	
	inform_give_reason.setSelected(check);
	accept.setSelected(check);
	confirm.setSelected(check);
	reject.setSelected(check);
	commit .setSelected(check);	
	offer.setSelected(check);
    }
    /**
     * @return the label of the JToggleButton that was selected
     * called by {@link #actionPerformed(ActionEvent) actionPerformed}
     */
    private String Check() {
	String value = new String("");
	if (greet.isSelected())
	    value = "greet!"+value;
	if (bye.isSelected())
	    value = "bye!"+value;
	if (introduce.isSelected())
	    value = "introduce!"+value;
	if (politeness_formula.isSelected())
	    value = "politeness_formula!"+value;
	if (thank.isSelected())
	    value = "thank!"+value;
	if (deliberate.isSelected())
	    value = "deliberate!"+value;
	if (backchannel.isSelected())
	    value = "backchannel!"+value;
	if (init.isSelected())
	    value = "init!"+value;
	if (defer.isSelected())
	    value = "defer!"+value;
	if (close.isSelected())
	    value = "close!"+value;
	if (request_suggest.isSelected())
	    value = "request_suggest!"+value;
	if (request_clarify.isSelected())
	    value = "request_clarify!"+value;
	if (request_comment.isSelected())
	    value = "request_comment!"+value;
	if (request_commit.isSelected())
	    value = "request_commit!"+value;
	if (suggest.isSelected())	
	    value = "suggest!"+value;
	if (inform_digress.isSelected())
	    value = "inform_digress!"+value;
	if (inform_exclude.isSelected())
	    value = "inform_exclude!"+value;
	if (inform_clarify.isSelected())
	    value = "inform_clarify!"+value;	
	if (inform_give_reason.isSelected())
	    value = "inform_give_reason!"+value;
	if (accept.isSelected())
	    value = "accept!"+value;
	if (confirm.isSelected())
	    value = "confirm!"+value;
	if (reject.isSelected())
	    value = "reject!"+value;
	if (commit .isSelected())
	    value = "commit!"+value;	
	if (offer.isSelected())
	    value = "offer!"+value;
	return value;
    }
    /**
     * displays the next turn with the corresponding dialog act properties. 
     * The next turn is selected from the input or output depending on the pressed button
     */
    public void changed(Judgements j) {
	String value = new String("no value");
	//this resets everything
	Check(false);
	//check the source
	if (j!=null){
	    if (input.isSelected()) {
		//check the tag
		value = (String) j.turn.get("DIALIN");
		if (value != null) {
		    StringTokenizer tok = new StringTokenizer(value, "!");
	
		    while (tok.hasMoreTokens()) {
			value = tok.nextToken();
			if (tok != null)
			    Check(value);
		    }
		}
		if((String)j.turn.get("IN")!=null)
		    turn.setText(ui.Options.substitute((String)j.turn.get("IN")));
		else 
		    turn.setText("");
	    }
	    else {
		// check the tag
		value = (String) j.turn.get("DIALOUT");
		if (value != null) {
		    StringTokenizer tok = new StringTokenizer(value, "!");
		    while (tok.hasMoreTokens()) {
			value = tok.nextToken();
			if (tok != null)
			    Check(value);
		    }
		}
		if( (String)j.turn.get("OUT")!=null)
		    turn.setText(ui.Options.substitute((String) j.turn.get("OUT")));
		else 
		    turn.setText("");	    
	    }
	}
       
    }

    /**
     * Enables actions on the JToggleButtons if the option "Edit data" in the main menu is validated
     * @see MyMenu#checkedit
     */
    public void Enable(boolean edit) {
	// CONTROL DIALOGUE
	greet.setEnabled(edit);
	bye.setEnabled(edit);
	introduce.setEnabled(edit);
	politeness_formula.setEnabled(edit);
	thank.setEnabled(edit);
	deliberate.setEnabled(edit);
	backchannel.setEnabled(edit);
	// MANAGE TASK
	init.setEnabled(edit);
	defer.setEnabled(edit);
	close.setEnabled(edit);
	// PROMOTE TASK
	request_suggest.setEnabled(edit);
	request_clarify.setEnabled(edit);
	request_comment.setEnabled(edit);
	request_commit.setEnabled(edit);
	suggest.setEnabled(edit);
	inform_digress.setEnabled(edit);
	inform_exclude.setEnabled(edit);
	inform_clarify.setEnabled(edit);
	inform_give_reason.setEnabled(edit);
	accept.setEnabled(edit);
	confirm.setEnabled(edit);
	reject.setEnabled(edit);
	commit.setEnabled(edit);
	offer.setEnabled(edit);
    }
}

	
