import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;


/**
 * This class builts  the model of the application
 * @see Tags
 * @see UserOptions 
 */
public class Document implements Tags, UserOptions { 
    /**
     * the file associated with the document
     */
    File file;
    /**
     * the number corresponding to the module which performed the translations
     */
    int output = 1;
    /**
     * first turn of the dialogue
     */
    public Judgements FirstTurn;
    /**
     * current turn of the dialogue
     */
    public Judgements CurrentTurn;
    /**
     * used for the adaptors connected with this model
     */
    public EventListenerList changeListeners = new EventListenerList();
    /**
     * number of turns in the dialogue
     */
    public int turn_number;
    /**
     * number of the current turn being used
     */
    public int current_turn_number;
    /**
     * holds the autosave property
     */
    boolean autosave = false;
    /**
     * indicates if the document is saved or not
     */
    boolean issaved = true;
    /**
     * indicates if every turn is saved or only some selected ones
     */
    boolean selsave = false;
    /**
     * gives the step according to which the turns are saved
     */
    int frequency_to_save = FREQUENCY_SAVE_TURNS;
    /**
     * here the data of the turn are contained
     */
    public Judgements j;
    /**
     * contains the number of bytes in the file
     */
    long elem_file=0;
    /**
     * number of bytes that have been read
     */
    int read_quant=0;
   

    /**
     * @return a referrence to the current object
     */
    public Document getDoc() {
	return this;
    }

    /**
     * @return the number of turns in the dialogue
     */
    public int getTurnNumber() {
	return turn_number;
    }

    /**
     * @return the number of the current turn
     */
    public int getCurrentTurnNumber() {
	return current_turn_number;
    }

    /**
     * set the step woith which the turns are saved
     * @param n the step
     */
    public void setFrequencytoSave(int n){
	frequency_to_save=n;
    }

    /**
     * @return the frequency to save the turns
     */
    public int getFrequencytoSave(){
	return frequency_to_save;
    }
    /**
     * @return first turn of the dialogue
     * @see Judgements
     */
    public Judgements getFirstTurn(){
	return FirstTurn;
    }


    /**
     * @param file the file associated with the current document
     */
    public Document(File file) {
	turn_number = 0;
	current_turn_number = 0;
	this.file=file;
    }


    /**
     * finds a turn according to the parameter
     * @param turn the number of the turn to be found
     * @see Judgements
     * @see #fireChange()
     */
    public void findTurn(int turn) {
	//	set Current Turn
	if (turn < 1)
	    turn = 1;
	if (turn > turn_number)
	    turn = turn_number;
	current_turn_number = turn;
	CurrentTurn = FirstTurn;
	for (int i = 0; i<turn-1; i++) {
	    CurrentTurn = (Judgements) CurrentTurn.turn.get("NEXT");
	}
	//update the UserInterface
	fireChange();
    }


    /**
     * retrieves the data of the current turn
     * @see Judgements
     */
    public Judgements getData() {
	return CurrentTurn;
    }

    /**
     * retrieves the data of the first turn of the dialogue
     * @see Judgements
     */
    public Judgements getFirst() {
	return FirstTurn;
    }

    /**
     * sets the values when no document is used
     * @see Judgements
     */
    public void ResetValues() {
	j = new Judgements(NO_TURNS, 1,this); 
	FirstTurn = j;
	CurrentTurn = FirstTurn;
	turn_number = 0;
	current_turn_number = 0;
    }

  
    /**
     * this method adds a change listener to the model
     */
    public void addChangeListener(ChangeListener x) {
	changeListeners.add(ChangeListener.class, x);
    }


    /**
     * this method removes a change listener to the model
     */
    public void removeChangeListener(ChangeListener x) {
	changeListeners.remove(ChangeListener.class, x);
    }

    /**
     * when a change occurs stateChanged is alled on the registered classes
     */
    protected void fireChange() {
	// create the event
	ChangeEvent c = new ChangeEvent(this);
	// Get the listener list
	Object[] listeners = changeListeners.getListenerList();
	// Process the listeners last to first
	// lsit in paris, class and instance
	for (int i = listeners.length-2; i>=0; i-= 2) {
	    if (listeners[i] == ChangeListener.class) {
		ChangeListener cl = (ChangeListener) listeners[i+1];
		cl.stateChanged(c);
	    }
	}
    }
  

    /** 
     * This method is the implementation of the tags interface add verifies 
     * wheter a string is in the tag[] array range or not. 
     * @param value indicates the string to look for in the tag[] array
     */
    public boolean isTag(String value) {
	int tag_number = Integer.parseInt(tag[0]);
	for (int i = 0; i < tag_number; i++)
	    if (tag[i].equals(value))
		return true;
	return false;
    }
}

