
import java.awt.*;
import java.awt.Color;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;

/**
 * This class creates the interface of the application by adding a frame, 
 * a menu, the toolbars and the user interface; a window listener is added so that the 
 * application can be closed thorugh the window manager of the OS.
 * @see MyMenu
 * @see UserInterface
 * The interface is size with the pack() method and 
 * then made visible with setVisible()
*/
public class MainFrame extends JFrame {
   
    /**
     * instance of the UserInterface class. Built in {@link #MainFrame() MainFrame}
     * @see UserInterface
     */
    private UserInterface ui;
    /**
     * instance of MyMenu class. Constructs the menu of the main window. 
     * Built in {@link #MainFrame() MainFrame}
     * @see MyMenu
     */
    private MyMenu menu;
    
  
    
    /**
       This method returns the user interface of the application
    */
    public UserInterface getUserInterface() {
        return ui;
    } 
    
    /**
       This method returns the menu of the application
    */
    public MyMenu getMenu() {
        return menu;
    } 


    /**
       The constructor of the application instantiates three objects: 
       a frame, a menu, and the user interface. Each object is a private 
       member and can be accessed through the corresponding public method
       * @see MyMenu#getToolBarActions()
       * @see #getUserInterface()
       */
    public MainFrame(){	
	//mainframe=new JFrame("GET");
	super("GET");
	Container c = this.getContentPane();
	c.setLayout(new BorderLayout());

	// create a menu and add a Menu Bar to the frame
        menu = new MyMenu(this);
	menu.EnableMenu(false);
	this.setJMenuBar(menu.getMenuBar()); 

	//add the toolbars to the frame
	c.add(menu.getToolBarActions(),BorderLayout.NORTH);
	

	// build the GUI
	ui = new UserInterface(this, menu);
	c.add(ui, BorderLayout.CENTER);

	// Create a WindowAdapter so the application 
	// is exited when the window is closed.
	this.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    });


	// pack the components
	this.pack();

	// make window visible
        this.setVisible(true);
    }
}
