import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;

/**
 * This class constructs the menu of the menu of the main window of the application.
 *  It implements the ActionListener interface in order to make the application able 
 * to respond to the actions from the menu
 */
public class MyMenu implements ActionListener, ComponentListener {
    /**
     * holds the menubar
     * @see #getMenuBar()
     */
    JMenuBar menubar;
    
    /**
     * holds the toolbar for the actions in the "File" menu, like Save, close, Open, etc
     * as well as for the other windows related to the main appication: 
     * Translation Panel, Recognized Panel etc;
     * @see #getToolBarActions() 
     */
    JToolBar toolbarActions;
   
    
    /**
     * used for constructing the main menu components
     * @see #MyMenu(MainFrame)
     */
    JMenu menu;
    /**
     * used for constructing the sub-menu "Options"
     * @see #MyMenu(MainFrame)
     */
    JMenu options;
   
    /**
     * holds the reference to the "Replace characters" item in the Options sub-menu 
     * of the Settings menu
     */
    JMenuItem change_char;
    /**
     * holds the referrence to the "Save every..." option in the Options sub-menu 
     * in the Settings menu
     */
    JMenuItem change_frequency;
    /**
     * used for constructing the "no files" item in the Documents menu
     */
    JMenuItem item_spec;
 
  
      /**
     * holds the reference to the "save" action related to the menu but also the toobarActions
     */
    MenuAction saveAction;
    /**
     * holds the reference to the "open" action related to the menu but also the toobarActions
     */
    MenuAction openAction;
    /**
     * holds the refference to the "save as" action related to the menu but also the toobarActions
     */
    MenuAction saveAsAction;
    /**
     * holds the reference to the reload action related to the menu but also the toobarActions
     */
    MenuAction reloadAction;
    /**
     * holds the reference to the close action related to the menu but also the toobarActions
     */
    MenuAction closeAction;
    /**
     * holds the reference to the quit action related to the menu but also the toobarActions
     */
    MenuAction quitAction;
    /**
     * holds the referrence to the action dealing with opening the Translation Window, 
     * done from the menu and also from the toolbarWindows
     */
    MenuAction translationWindow;
    /**
     * holds the reference to the action dealing with opening the Recognizer Panel,
     *  done from the menu and also from the toolbarWindows
     */
    MenuAction recognizedInput;
    /**
     * holds the reference to the action dealing with opening the Dialog Act,
     *  done from the menu and also from the toolbarWindows
     */
    MenuAction dialogAct;
    /**
     * holds the reference to the action dealing with opening the Pattern Window,
     *  done from the menu and also from the toolbarWindows
     */
    MenuAction pattern;
    /**
     * holds the reference to the action dealing with opening the Judgements Statistics Window,
     *  done from the menu and also from the toolbarWindows
     */
    MenuAction judgements;
    /**
     * holds the reference to the action dealing with opening the Translation Statistics Window, 
     * done from the menu and also from the toolbarWindows
     */
    MenuAction translations;
    /**
     * holds a referrence to the "Edit data" option in the Settings menu
     */
    JCheckBoxMenuItem checkedit;
    /**
     * holds a referrence to the "Selective Save" option in the Settings menu
     */
    JCheckBoxMenuItem automaticCompar;
    /**
     * holds a referrence to the "Autosave" option in the Settings menu
     */
    JCheckBoxMenuItem checksave;
    /**
     * holds a referrence to the "Original translation" option in the Translations sub-menu
     *  from the View menu
     */
    JCheckBoxMenuItem checkoriginal;
    /**
     * holds a referrence to the "Case-based translation" option in the Translations sub-menu 
     * from the View menu
     */
    JCheckBoxMenuItem checkali;
    /**
     * holds a referrence to the "Statistical translation" option in the Translations sub-menu 
     * from the View menu
     */
    JCheckBoxMenuItem checkstat;
    /**
     * holds a referrence to the "Dialog-based translation" option in the Translations sub-menu
     *  from the View menu
     */
    JCheckBoxMenuItem checksyn;
    /**
     * holds a referrence to the "Deep Analysis" option in the Translations sub-menu
     * from the View menu
     */
    JCheckBoxMenuItem checkdeep;
    /**
     * holds a referrence to the "Preselection" option in the Translations 
     * sub-menu from the View menu
     */
    JCheckBoxMenuItem checkpre;
    
    /**
     * holds a reference to the UserInterface. It is instantiated only in actionPerformed method
     * and not in the constructor as the menu and the User Interface are created in the
     * same object (MainFrame)
     * @see #actionPerformed(ActionEvent)
     * @see MenuAction#actionPerformed(ActionEvent)
     */
    UserInterface ui;
    /**
     * holds a referrence to the MainFrame class
     * @see MainFrame 
     */
    MainFrame application;
    /**
     * contains the name of the opened documents. It is used in the Document menu
     * @see #addFile(String)
     * @see #updateInClose(String)
     * @see #removeFile(String)
     */ 
    Vector doc_items;
    /**
     * used for identifying the command for the menu
     * @see MenuAction#actionPerformed(ActionEvent)
     * @see #actionPerformed(ActionEvent) 
     */
    String command;

    /**
     * Returns the menubar field
     * @see MainFrame#MainFrame()
     */
    public  JMenuBar getMenuBar(){
	return menubar;
    }
    /**
     * Returns the first toolbar, concerning the actions in the File menu
     * @see MainFrame#MainFrame()
     */
    public  JToolBar getToolBarActions(){
	return toolbarActions;
    }

    /**
     * Depending on the value of the boolean parameter the method enables or disables 
     * the operations to be done on a document as opening, closing, saving etc.
     * @param enable is  false if no document is opened 
     * @see UserInterface#Close()
     * @param enable is true if at least one document is opened
 
     */
    public  void EnableMenu(boolean enable) {
	if (enable) {
	    //enables actions in File menu
	    saveAction.setEnabled(true);
	    saveAsAction.setEnabled(true);
	    reloadAction.setEnabled(true);
	    closeAction.setEnabled(true);
	    //enables options in the Settings menu
	    menubar.getMenu(1).getItem(0).setEnabled(true);
	    menubar.getMenu(1).getItem(1).setEnabled(true);
	    menubar.getMenu(1).getItem(2).setEnabled(true);
	    menubar.getMenu(1).getMenuComponent(3).setEnabled(true);
	    //enables actions and options in the View menu
	    menubar.getMenu(2).getMenuComponent(0).setEnabled(true);
	    translationWindow.setEnabled(true);
	    recognizedInput.setEnabled(true);
	    dialogAct.setEnabled(true);
	    //enables actions in the Statistics menu
	    pattern.setEnabled(true);
	    judgements.setEnabled(true);
	    translations.setEnabled(true);
	}
	else { 
	    //disables actions in the file menu
	    saveAction.setEnabled(false);
	    saveAsAction.setEnabled(false);
	    reloadAction.setEnabled(false);
	    closeAction.setEnabled(false);
	    //disables options in the Settings menu
	    menubar.getMenu(1).getItem(0).setEnabled(false);
	    menubar.getMenu(1).getItem(1).setEnabled(false);
	    menubar.getMenu(1).getItem(2).setEnabled(false);
	    menubar.getMenu(1).getMenuComponent(3).setEnabled(false);
	    //disables actions in the View menu
	    menubar.getMenu(2).getMenuComponent(0).setEnabled(false);
	    translationWindow.setEnabled(false);
	    recognizedInput.setEnabled(false);
	    dialogAct.setEnabled(false);
	    //disables actions in the Statistics menu
	    pattern.setEnabled(false);
	    judgements.setEnabled(false);
	    translations.setEnabled(false);
	  
	}
    }

	
    /**
     * adds a new menu item to the menu Documents whenever a new file is opened
     * shows that this is the current document by seting the state of the current 
     * item to true and of the others to false if a new JCheckBoxMenuItem is created 
     * the name of this item is added to the Vector doc_items if the document is only 
     * reloaded it sets only the states to the corresponding values ui.newdocument is 
     * true if a new document is opened and false otherwise 
     * @see UserInterface#Open(JComponent)
     * @param s the name of the file to be added
     */
    public void addFile(String s){	   
	if(ui.new_document) {
	    //used when closing all documents
	    menu = menubar.getMenu(menubar.getMenuCount()-1);
	    if(s.equals("no files")) {
		menu.add(item_spec);
		return;
	    }
	    //adds a new item to the menu and removes "no files"
	    JCheckBoxMenuItem item1 = new JCheckBoxMenuItem (s);
	    menu.add (item1);
	    item1.setState(true);
	    item1.addActionListener (this);
	    if (menu.getItem(0) == item_spec) 
		menu.remove(item_spec);
	    update();
	    doc_items.add(s);
	}
	//if another file from the already opened documents is displayed
	else if (menu.getItemCount()>0) {
	    int pos = menu.getItemCount();
	    for(int k = 0; k < pos; k++) 
		((JCheckBoxMenuItem)menu.getItem(k)).setState(false);
	    ((JCheckBoxMenuItem)menu.getItem(doc_items.indexOf(s))).setState(true);
	}
    }

    /**
     * sets the items in the Document menu as not selected
     */
    public void update(){
	menu = menubar.getMenu(menubar.getMenuCount()-1);
	if (menu.getItemCount() > 1) {
	    int pos = menu.getItemCount();
	    for(int k = 0; k < pos - 1; k++) 
		((JCheckBoxMenuItem)menu.getItem(k)).setState(false);
	}
    }
    /**
     * updates the Document Menu by setting by opened the document opened before the one closed
     * @param s the name of the previous opened document
     * @see UserInterface#Close()
     */
    public void updateInClose(String s){
	menu=menubar.getMenu(menubar.getMenuCount()-1);
	( (JCheckBoxMenuItem)menu.getItem(doc_items.indexOf(s))).setState(true);
    }
    /**
     * removes an item from the Document menu, whenever a document is close
     * It also removes the name of the item from the doc_items Vector
     * @see UserInterface#Save()
     */
    public void removeFile(String s){
	menu=menubar.getMenu(menubar.getMenuCount() - 1);
	int index = doc_items.indexOf(s);
	if (index != -1) {
	    menu.remove(doc_items.indexOf(s));
	    doc_items.remove(s);
	}
    }

  
    /**
     * The parameter is an instance of the MainFrame class, needed for getting access to the User interface
     * @see MainFrame
     */
    public MyMenu(MainFrame application) {
	JMenuItem item;	
	// creates menubar
	this.application=application;
	doc_items=new Vector();
	menubar = new JMenuBar();

	// creates file menu
	menu = new JMenu ("File");
	menu.setMnemonic('F');
	openAction = new MenuAction("Open",new ImageIcon("Open.gif"));
	item = menu.add(openAction);
	item.setIcon((Icon)openAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('O', java.awt.Event.ALT_MASK, false));

	saveAction = new MenuAction("Save",new ImageIcon("Save.gif"));
	item = menu.add(saveAction);
	item.setIcon((Icon)saveAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('S', java.awt.Event.ALT_MASK, false));

	saveAsAction = new MenuAction("Save as...",new ImageIcon("SaveAs.gif"));
	item = menu.add(saveAsAction);
	item.setIcon((Icon)saveAsAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('S', java.awt.Event.CTRL_MASK, false));

	reloadAction = new MenuAction("Reload",new ImageIcon("Reload.gif"));
	item = menu.add(reloadAction);
	item.setIcon((Icon)reloadAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('R', java.awt.Event.ALT_MASK, false));

	closeAction = new MenuAction("Close",new ImageIcon("Close.gif"));
	item = menu.add(closeAction);
	item.setIcon((Icon)closeAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('C', java.awt.Event.ALT_MASK, false));
	menu.addSeparator();


	quitAction = new MenuAction("Quit",new ImageIcon("Quit.gif"));
	item = menu.add(quitAction);
	item.setIcon((Icon)quitAction.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('Q', java.awt.Event.ALT_MASK, false));
	menubar.add (menu);
	
	// creates settings menu
	menu = new JMenu ("Settings");
	menu.setMnemonic('S');
	menu.add( checksave = new JCheckBoxMenuItem ("Autosave"));
	checksave.setAccelerator(KeyStroke.getKeyStroke('A', java.awt.Event.CTRL_MASK, false));
	checksave.addActionListener (this);

	menu.add( automaticCompar = new JCheckBoxMenuItem ("Automatic Comparison"));
	automaticCompar.setAccelerator(KeyStroke.getKeyStroke('C', java.awt.Event.SHIFT_MASK, false));
	automaticCompar.addActionListener (this);

	menu.add( checkedit = new JCheckBoxMenuItem ("Edit data"));
	checkedit.setAccelerator(KeyStroke.getKeyStroke('E', java.awt.Event.CTRL_MASK, false));
	checkedit.addActionListener (this);
	options=new JMenu("Options");
	options.add(change_char =new JMenuItem("Replace characters"));
	change_char.addActionListener(this);

	options.add(change_frequency =new JMenuItem("Save turns every..."));
	change_frequency.addActionListener(this);
	menu.add(options);
	menubar.add (menu);

	// creates view menu
	JMenu menu = new JMenu ("View");
	menu.setMnemonic('V');
	JMenu menuitem;
	ButtonGroup gr = new ButtonGroup();
	menu.add( menuitem = new JMenu("Translation"));
	menuitem.add( checkoriginal = new JCheckBoxMenuItem ("Original selection"));
	checkoriginal.addActionListener (this);
	checkoriginal.setSelected(true);
	gr.add(checkoriginal);
	menuitem.add( checkali = new JCheckBoxMenuItem ("Case based translation"));
	checkali.addActionListener (this);
	gr.add(checkali);
	menuitem.add( checkstat = new JCheckBoxMenuItem ("Statistical translation"));
	checkstat.addActionListener (this);
	gr.add(checkstat);
	menuitem.add( checksyn = new JCheckBoxMenuItem ("Dialog based translation"));
	checksyn.addActionListener (this);
	gr.add(checksyn);
	menuitem.add( checkdeep = new JCheckBoxMenuItem ("Deep analysis"));
	checkdeep.addActionListener (this);
	gr.add(checkdeep);
	menuitem.add( checkpre = new JCheckBoxMenuItem ("Preselection"));
	checkpre.addActionListener (this);
	gr.add(checkpre);

	translationWindow = new MenuAction("Translations Window",new ImageIcon("TranslationsWindow.gif"));
	item = menu.add(translationWindow);
	item.setIcon((Icon)translationWindow.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('T', java.awt.Event.ALT_MASK, false));

        recognizedInput = new MenuAction("Recognized input",new ImageIcon("RecognizedInput.gif"));
        item = menu.add(recognizedInput);
	item.setIcon((Icon)recognizedInput.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('R', java.awt.Event.ALT_MASK, false));

	dialogAct = new MenuAction("Dialog act",new ImageIcon("DialogAct.gif"));
	item = menu.add(dialogAct);
	item.setIcon((Icon)dialogAct.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('D', java.awt.Event.ALT_MASK, false));
	menubar.add (menu);

	// creates Statistics menu
	menu = new JMenu ("Statistics");
	menu.setMnemonic('S');
	pattern = new MenuAction("Pattern",new ImageIcon("Pattern.gif"));
	item = menu.add(pattern);
	item.setIcon((Icon)pattern.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('P', java.awt.Event.CTRL_MASK, false));

	judgements = new MenuAction("Judgements",new ImageIcon("Judgements.gif"));
	item = menu.add(judgements);
	item.setIcon((Icon)judgements.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('J', java.awt.Event.CTRL_MASK, false));

	translations = new MenuAction("Translations",new ImageIcon("Translations.gif"));
	item = menu.add( translations);
	item.setIcon((Icon)translations.getValue(Action.SMALL_ICON));
	item.setHorizontalTextPosition(JMenuItem.RIGHT);
	item.setAccelerator(KeyStroke.getKeyStroke('T', java.awt.Event.CTRL_MASK, false));
	menubar.add (menu);

	//creates Documents Menu
	menu = new JMenu("Documents");
	menu.setMnemonic('D');
	menu.add(item_spec = new JMenuItem ("no files"));
	menubar.add(menu);

	toolbarActions = new JToolBar();
	toolbarActions.addComponentListener(this);
	toolbarActions.setMaximumSize(toolbarActions.getSize());
	JButton button;
	button=toolbarActions.add(openAction);
       	button.setActionCommand((String)openAction.getValue(Action.NAME));
	button.setToolTipText((String)openAction.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(saveAction);
	button.setActionCommand((String)saveAction.getValue(Action.NAME));
	button.setToolTipText((String)saveAction.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(saveAsAction);
	button.setActionCommand((String)saveAsAction.getValue(Action.NAME));
	button.setToolTipText((String)saveAsAction.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(reloadAction);
	button.setActionCommand((String)reloadAction.getValue(Action.NAME));
	button.setToolTipText((String)reloadAction.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(closeAction);
	button.setActionCommand((String)closeAction.getValue(Action.NAME));
	button.setToolTipText((String)closeAction.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(quitAction);
	button.setActionCommand((String)quitAction.getValue(Action.NAME));
	button.setToolTipText((String)quitAction.getValue(Action.NAME));
	button.setText("");

	toolbarActions.addSeparator();

	button=toolbarActions.add(translationWindow);
	button.setActionCommand((String)translationWindow.getValue(Action.NAME));
	button.setToolTipText((String)translationWindow.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(recognizedInput);
	button.setActionCommand((String)recognizedInput.getValue(Action.NAME));
	button.setToolTipText((String)recognizedInput.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(dialogAct);
	button.setActionCommand((String)dialogAct.getValue(Action.NAME));
	button.setToolTipText((String)dialogAct.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(pattern);
	button.setActionCommand((String)pattern.getValue(Action.NAME));
	button.setToolTipText((String)pattern.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(judgements);
	button.setActionCommand((String)judgements.getValue(Action.NAME));
	button.setToolTipText((String)judgements.getValue(Action.NAME));
	button.setText("");

	button=toolbarActions.add(translations);
	button.setActionCommand((String)translations.getValue(Action.NAME));
	button.setToolTipText((String)translations.getValue(Action.NAME));
	button.setText("");
    }
    /**
     * @param save gives information about the current status of the document (if it was saved or not)
     * Depending on this it enables the Save option in the File menu
     * @see Document
     * @see UserInterface
     */
    public void setSavable(boolean save) {

	ui=application.getUserInterface();
	if (save == true) {
	    ui.getCurrentDocument().issaved = false;
	    saveAction.setEnabled(true);
	}
	else {
	    ui.getCurrentDocument().issaved = true;
	    saveAction.setEnabled(false);	    
	}
    }
    /**
     * starts a thread for the actions in the menu
     */
    public void actionPerformed (ActionEvent e) {
	command = e.getActionCommand();
	final SwingWorker worker = new SwingWorker() {
		public Object construct() {
		    ActionFromMenu();
		    return command;
		}
	    };
	worker.start();
    }

    public void componentHidden(ComponentEvent e) {
    }
    
    public void componentMoved(ComponentEvent e) {
    }
    
    public void componentResized(ComponentEvent e) {
    	application.pack();
    }
    
    public void componentShown(ComponentEvent e) {
    }

    /**
     * passes to the UserInterface the commands from the menu
     * @see UserInterface#EditData(boolean)
     * @see UserInterface#setAutosave(boolean)
     * @see UserInterface#setOutput(int)
     * @see Document#fireChange()
     * @see TranslationPanel#ShowDialog()
     */
    private void ActionFromMenu() {
	this.ui=application.getUserInterface();
	if (command.equals("Edit data"))
	    ui.EditData(checkedit.getState());
	else if (command.equals("Autosave")){
	    ui.setAutosave(checksave.getState());
	    if (checksave.getState())
		JOptionPane.showMessageDialog(application,"Automatic save when changing the turn","Click OK",JOptionPane.INFORMATION_MESSAGE);
	}
	else if (command.equals("Automatic Comparison")){
	    ui.setAutomaticComparison(automaticCompar.getState());
	    if (automaticCompar.getState())
		JOptionPane.showMessageDialog(application,"Automatic comparison of original turns and recognized turns","Click OK",JOptionPane.INFORMATION_MESSAGE);
	}
	else if (command.equals("Replace characters")) {		
	    ui.repl_char.ShowDialog();
	}
	else if (command.equals("Save turns every...")) {
	    ui.change_freq.ShowDialog();
	}
	else if (command.equals("Original selection")) {
	    ui.setOutput(1);
	    ui.getCurrentDocument().fireChange();
	}
	else if (command.equals("Case based translation")) {
	    ui.setOutput(2);
	    ui.getCurrentDocument().fireChange();
	}
	else if (command.equals("Statistical translation")) {
	    ui.setOutput(3);
	    ui.getCurrentDocument().fireChange();
	}
	else if (command.equals("Dialog based translation")) {
	    ui.setOutput(4);
	    ui.getCurrentDocument().fireChange();
	}
	else if (command.equals("Deep analysis")) {
	    ui.setOutput(5);
	    ui.getCurrentDocument().fireChange();
	}
	else if (command.equals("Preselection")) {
	    ui.setOutput(6);
	    ui.getCurrentDocument().fireChange();
	}
	// actions related to the Document menu
	else if ((command.substring(command.length()-4)).equals("eval")) {
	    // get the file name
	    ui.setDocument(command);
	    ui.Redisplay();
	    //ui.setDocument((Document) ui.docs.elementAt(ui.docs_names.indexOf(command)));
	    //ui.Redisplays((Document) ui.docs.elementAt(ui.docs_names.indexOf(command)));
	    setSavable(!ui.getCurrentDocument().issaved);
	    //sets the chosen document to be displayed 
	    if (menu.getItemCount()>=1) {
		int pos=menu.getItemCount();
		for(int k=0;k<pos;k++) 	
		    ((JCheckBoxMenuItem)menu.getItem(k)).setState(false);
		((JCheckBoxMenuItem)menu.getItem(doc_items.indexOf(command))).setState(true);
	    }		
	}
    }


    /**
     * class  necessary for those actions which are retrieved also on the toolbars.
     * The extension of the AbstractAction class allows identical behaviour of a 
     * specified menu item and its associated button on the toolbar
     */
    class MenuAction extends AbstractAction{
	String command;
	UserInterface ui;
	public MenuAction(String text){
	    super(text);
	}
	/**
	 * Constructor used when to the action we associate also an icon
	 * @param text the name of the action
	 * @param icon the icon associated. It is an image in GIF format
	 */
	public MenuAction(String text, Icon icon){
	    super(text,icon);
	}

	/**
	 * captures the event and intialize a separate thread to process it. 
	 * The thread is intilaized using the SwingWorker class
	 */
	public void actionPerformed (ActionEvent e) {
	    command = e.getActionCommand();
	    final SwingWorker worker = new SwingWorker() {
		    public Object construct() {
			ActionFromMenu();
			return command;
		    }
		};
	    worker.start();
	}

	/**
	 * passes to the UserInterface the commands from the menu
	 * @see UserInterface#Open(JComponent)
	 * @see UserInterface#Save()
	 * @see RecognizerPanel#ShowDialog()
	 * @see DialogActPanel#ShowDialog()
	 * @see DialogPattern#ShowDialog()
	 * @see DialogStatistics#ShowDialog()
	 * @see TranslationStatistics#ShowDialog()
	 */
	private void ActionFromMenu() {
	    ui=application.getUserInterface();
	    if (command.equals("Open"))
		ui.Open((JComponent) menubar.getParent());
	    else if (command.equals("Save"))
		ui.Save();
	    else if (command.equals("Save as..."))
		ui.SaveAs((JComponent) menubar.getParent());
	    else if (command.equals("Reload"))
		ui.Reload();
	    else if (command.equals("Close"))
		ui.Close();
	    else if (command.equals("Quit"))
		ui.Quit();
	    else if (command.equals("Translations Window"))
		ui.translation_dialog.ShowDialog();
	    else if (command.equals("Recognized input"))
		ui.recognizer_dialog.ShowDialog();
	    else if (command.equals("Dialog act"))
		ui.dialog_dialog.ShowDialog();
	    else if (command.equals("Pattern"))
		ui.dialog_pattern.ShowDialog();
	    else if (command.equals("Judgements"))
		ui.dialog_judgements.ShowDialog();
	    else if (command.equals("Translations"))
		ui.dialog_stat_trans.ShowDialog();
	}

    }    
}
