import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import javax.swing.border.*;
/**
 * This class builds the user interface of the application 
 * and manages the events occuring when opening or closing a document.
 * @see UserOptions
 */
public class UserInterface extends JPanel implements UserOptions {
    /**
     * holds a reference to the current showed document
     */
    private Document curr_doc;
    /**
     * holds a reference to the frame of the application. 
     * Used as parrent for any other window built while the application runs
     * @see #UserInterface(JFrame,MyMenu)
     */
    JFrame mainframe;
    /**
     * holds a reference to the main menu of the application
     */
    MyMenu menu;
    /**
     * reference to the "Translations Window" panel
     */
    TranslationPanel translation_dialog;
    /**
     * reference to the "Recognizer Input" panel
     */
    RecognizerPanel recognizer_dialog;
    /**
     * reference to the "Dialog act" panel
     */
    DialogActPanel dialog_dialog;
    /**
     * reference to the "Judgements Statististics" panel
     */
    DialogStatistics dialog_judgements;
    /**
     * reference to the "Translations Statistics" panel
     */
    TranslationStatistics dialog_stat_trans;
    /**
     * reference to the "Patterns" panel
     */
    DialogPattern dialog_pattern;
    /**
     * reference to the panel containing the current analysed turn
     */
    CurrentTurnPanel ct;
    /**
     * reference to the panel containing informations about the current turn panel
     */
    TurnCorrectnessPanel tc;
    /**
     * reference to the panel containing the next turn in the dialogue
     */
    NextTurnPanel nt;
    /**
     * reference to the panel Information elements, which shows also 
     * the number of the current turn dispayed
     */
    InformationElementsPanel ie;
    /**
     * reference to the window "Replace characters", accessed from the options sub-menu, 
     * from Settings menu
     */
    ReplaceCharacters repl_char;
    /**
     * reference to the window "Save every...", popped-up from the options sub-menu, 
     * from Settings menu
     */
    ChangeFrequency change_freq;
    /**
     * reference to the object which performs the changes of the characters to be replaced
     */
    OptionsString Options;
    /**
     * contains the current opened docs
     */
    Vector docs;
    /**
     * contains the names of the current opened docs
     */
    Vector docs_names;
    /**
     * indicates if the current opened document is a new one, or it was opened earlier
     */
    boolean new_document;
    /**
     * indicated the way to perform the comparison in the Recognizer Panel window
     */
    boolean autom_compar;
    
    /**
     * this object loads a document
     */
    FileLoader loader;
    /**
     * this object saves a document
     */
    FileSaver saver;
    /**
     * used for starting the progress monitor
     */
    javax.swing.Timer timer;

    /**
     * @return the reference to the user interface
     */
    public UserInterface getUI(){
	return this;
    }

    /**
     * @return a reference to the current opened document, if there is one
     */
    public Document getCurrentDocument(){
	if(curr_doc!=null)	   
	    return curr_doc;
	return (Document) null;
    }

    /**
     * @return the document with a specified name
     * @param name the name of the document to be retieved
     * @see Document
     */
    private Document setCurrentDocument(String name) {
	for (int i = 0; i < docs.size(); i++)
	    if ((((Document) docs.elementAt(i)).file.getName()).equals(name))
		return (Document) docs.elementAt(i);
	return (Document) null;
    }
    /**
     * deletes the name of a document from the docs_names vector
     * @param name the string to be removed
     */
    
    private void updateDocsNames(String name) {
	/* delete the name of the file from the old position */
	for (int i = 0; i < docs_names.size(); i++)
	    if (((String) docs_names.elementAt(i)).equals(name)) {
		docs_names.removeElementAt(i);
		break;
	    }
	/* add the name in the last position */
	docs_names.addElement(name);
    }

    /**
     * sets the current opened document
     * @param name  the name of the  document to be set
     * @see #setCurrentDocument(String)
     */
    public void setDocument(String name) {
	/* retrieves the document to be displayed from its name */
	curr_doc = setCurrentDocument(name);
	/* updates the docs_names vector */
	updateDocsNames(name);
    }
    /**
     * @return reference to the InformationElementsPanel object. 
     * used for dispaying informations concerning the new opened file
     * @see #DisplayCurrentTurn()
     */
    public InformationElementsPanel getInformationElementsPanel(){
	return ie;
    }
    /**
     * sets the type of module that produced the translation
     * @param output indicates the module (each module has a number assigned) 
     */
    public void setOutput(int output){
	curr_doc.output=output;
    }

    /**
     *@param compar determines if in the Recognizer panel the comparison is done automatically
     */
    public void setAutomaticComparison(boolean compar){
	 autom_compar = compar;
	 if(autom_compar){
	     recognizer_dialog.compare.setEnabled(false);
	     recognizer_dialog.compare_algorithm();
	 }
	 else {
	     recognizer_dialog.compare.setEnabled(true);
	     recognizer_dialog.initial_state();
	 }
    }

    /**
     * @return true if the comparison in the recognizer panel is done automatically
     */
    public boolean getAutomaticCompare(){
	return autom_compar;
    }
    /**
     * Constructor of the user interface
     */
    public UserInterface(JFrame mainframe, MyMenu menu){
	this.menu=menu;
	this.mainframe=mainframe;


	//instantiate the main windows to be displayed
	repl_char=new ReplaceCharacters(this.mainframe,this);
	Options=new OptionsString(this); 
	change_freq = new ChangeFrequency(this.mainframe,this);
        dialog_stat_trans=new TranslationStatistics(this.mainframe, this);
	translation_dialog=new TranslationPanel(this.mainframe,this);
	recognizer_dialog=new RecognizerPanel(this.mainframe,this);
	dialog_dialog=new DialogActPanel(this.mainframe,this,curr_doc);
	dialog_judgements=new DialogStatistics(this.mainframe,this);
	dialog_pattern=new DialogPattern(this.mainframe);   
        //       set layout in the frame
	setLayout(new BorderLayout());
	// insert a panel
	JPanel subpanel = new JPanel(new BorderLayout());
	// adds the first component to the subpanel
        ct=new CurrentTurnPanel(this);
	subpanel.add(ct, BorderLayout.NORTH);	
        tc=new TurnCorrectnessPanel(this);
	subpanel.add(tc, BorderLayout.CENTER);
	nt = new NextTurnPanel(this);
	nt.setBorder(new EmptyBorder(16,20,25,20));
	subpanel.add(nt, BorderLayout.SOUTH);
	
	// builts the main panel
	add(subpanel, BorderLayout.NORTH);	
	ie = new InformationElementsPanel(this,this.mainframe);
	ie.setBorder(new EmptyBorder(16,20,25,20));
	add(ie, BorderLayout.SOUTH);
	// set the editability of the interface
	EditData(false);
	//initialize the vectors concerning the opened documents
	docs = new Vector();
	docs_names = new Vector();
    }

    /**
     * assigns the autosave property to the current document
     * @param auto is true if the "autosave" option in the Settings menu is selected
     * @see MyMenu#actionPerformed(ActionEvent)
     */

    public void setAutosave(boolean auto) {
	curr_doc.autosave = auto;
    }
   

    /**
     * enables changes of different components of the user interface
     * @param edit is true if the option "Edit Data" in the Settings menu is selected
     * @see MyMenu#actionPerformed(ActionEvent)
     */
    public void EditData(boolean edit) {
	ct.TranslationMismatchYes.setEnabled(edit);
	ct.TranslationMismatchNo.setEnabled(edit);
	ct.TranslationHuman.setEnabled(edit);
	ct.TranslationMachine.setEnabled(edit);
	ct.TranslationQualityGood.setEnabled(edit);
	ct.TranslationQualityIntermediate.setEnabled(edit);
	ct.TranslationQualityBad.setEnabled(edit);
	tc.SyntCorrInYes.setEnabled(edit);
	tc.SyntCorrInNo.setEnabled(edit);
	tc.SemCorrInYes.setEnabled(edit);
	tc.SemCorrInNo.setEnabled(edit);
	tc.MissInYes.setEnabled(edit);
	tc.MissInNo.setEnabled(edit);
	tc.SyntCorrOutYes.setEnabled(edit);
	tc.SyntCorrOutNo.setEnabled(edit);
	tc.SemCorrOutYes.setEnabled(edit);
	tc.SemCorrOutNo.setEnabled(edit);
	tc.MissOutYes.setEnabled(edit);
	tc.MissOutNo.setEnabled(edit);
	ie.IE.plus1.setEnabled(edit);
	ie.IE.minus1.setEnabled(edit);
	ie.IE.number.setEnabled(edit);
	ie.EIE.plus1.setEnabled(edit);
	ie.EIE.minus1.setEnabled(edit);
	ie.EIE.number.setEnabled(edit);
	ie.LIE.plus1.setEnabled(edit);
	ie.LIE.minus1.setEnabled(edit);
	ie.LIE.number.setEnabled(edit);
	ie.TIE.plus1.setEnabled(edit);
	ie.TIE.minus1.setEnabled(edit);
	ie.TIE.number.setEnabled(edit);
	ie.AIE.plus1.setEnabled(edit);
	ie.AIE.minus1.setEnabled(edit);
	ie.AIE.number.setEnabled(edit);
	ie.plus1.setEnabled(edit);
	ie.minus1.setEnabled(edit);
	ie.TN.setEnabled(edit);
	ie.TNs.setEnabled(edit);
	dialog_dialog.Enable(edit);
    }
    /**
     * enables or disables the browsing of the turns in the current document
     * @param browse is true when at least one document is opened and false otherwise
     * @see #Close()
     * @see #Open(JComponent)
     */
    public void BrowseData(boolean browse) {
	ie.plus1.setEnabled(browse);
	ie.minus1.setEnabled(browse);
	ie.TN.setEnabled(browse);
	ie.TNs.setEnabled(browse);	
    }

    /**
     * reloads from the disk the current document 
     */
    public void Reload(){
	// starts the loader
	loader = new FileLoader(curr_doc,mainframe);
	// replaces the previous opened file with the new one
	docs.setElementAt(curr_doc,docs.size()-1);
	docs_names.setElementAt(curr_doc.file.getName(),docs.size()-1);
    }


    /**
     * opens a file if it is a new one or redisplays an existent one
     * @param parent the parent frame for the dialog window which allows the selection of the file
     * @see #SelectFile(JComponent,char)
     * @see MyMenu#EnableMenu(boolean)
     * @see MyMenu#update()
     * @see MyMenu#setSavable(boolean)
     * @see #BrowseData(boolean)
     * @see FileLoader
     */
    public void Open(JComponent parent) {
	File file;
	file = SelectFile(parent,'o');
	//if the selection of the file was done for opening 
	if (file != (File) null) {
	    //if there is a new document to be opened
	    if (new_document) {
		//starts the loader
		loader = new FileLoader(curr_doc,mainframe);
		//if it is the first file
		if(docs.size() == 1) {
		    menu.EnableMenu(true);
		    BrowseData(true);
		}
		menu.setSavable(false);
		// marks the other items in the Document menu as not being the current opened docs
		menu.update();
	    }
	    else 
		//if the selected file is already opened
		Redisplay();
	    //adds the name of the new File to the Documents menu and mark it as the one in use
	    menu.addFile(curr_doc.file.getName());	    
	}
    }

    /**
     * Pops up a dialog window in order to select a file for opening or for saving under another name
     * @param parent the parent frame of the window
     * @param type is 'o' if the window is popped-up for opening and 's' if it is for saving
     * @return true if a file was selected
     * @see #verifyExtension(File)
     * @see #openNewFile(File)
     */
    private File SelectFile(JComponent parent, char type) {
	File file = (File)null;
	//pops-up the dialog window
	JFileChooser chooser = new JFileChooser();
	// shows only the files with the extention "eval"
	ExtensionFileFilter filter = new ExtensionFileFilter("eval");
	filter.setDescription("Evaluation Files");
	chooser.setFileFilter(filter);
	//sets the current directory
	chooser.setCurrentDirectory(new File("."));
	int ret = 0;
	// if the selected file should be opened 
	if(type=='o'){
	    // returnVal tells if the selection is aproved or discarded
	    ret = chooser.showOpenDialog(parent);
	}
	// if the window was poped-up for saving
	else if(type=='s'){
	    // returnVal tells if the selection is approved or discarded
	    ret = chooser.showSaveDialog(parent);
	}
	//if the selection was approved
	if(ret == JFileChooser.APPROVE_OPTION) {
	    //retrieve the name of the file
	    file = chooser.getSelectedFile();
	    //if the file was selected for opening
	    if(type=='o'){
		//if there is no file with this name in the already opened ones
		if (!docs_names.contains(file.getName())) {
		    //opens a new file
		    openNewFile(file);
		    return file;
		}
		else {
		    //opens an existent file i.e. redisplays it
		    openExistentFile(file);
		    return file;
		}
	    }
	    //if the selection was done for a saving
	    else if(type=='s') {
		//mark that a new document is created
		//verifies if it has the correct extension; if not the extension is added
		file = verifyExtension(file);
		new_document = true;
		return file;
	    }
	}
	// if the selection was not approved 
	return (File) null;
    }

    /**
     * checks if the selected file has the correct extension (eval)
     * if not this extention is added to the name of the file
     * @param file the name of the file for which te test is performed
     */
    private File verifyExtension(File file) {
	String filename = file.getName();
	StringBuffer path = new StringBuffer(file.getPath());
	path =new StringBuffer( path.substring(0, path.length() - filename.length()));
	//if the length of the file is less than file it is obvious that the extension does not exist
	if(filename.length()<5) 
	    filename = filename + ".eval";
	//if the name of the file does not end with eval
	else if(!(filename.substring(filename.length()-4).equals("eval")))
	    filename = filename + ".eval";

	path = path.append(filename);
	file = new File(path.toString());
	return file;
    }

    /**
     * created a new Document based on the current selected file
     * @see Document
     * @see #create_adaptors()
     */
    private void openNewFile(File file) {
	//creates a new Document object
	curr_doc = new Document(file);
	//created the adaptors for this document
	create_adaptors();
	//adds the file in the vectors managing the opened docs
	docs_names.addElement(file.getName());
	docs.addElement(curr_doc);
	//a new document is opened
	new_document = true;
    }

    /**
     * created the adaptors for a new document
     * @see ModelToCurrentTurnPanelAdaptor
     * @see ModelToInformationsElementPanelAdaptor
     * @see ModelToTurnCorectnessPanelAdaptor
     * @see ModelToNextTurnPanelAdaptor
     * @see ModelToTranslationPanelAdaptor
     * @see ModelToRecognizerPanelAdaptor
     * @see ModelToDialogActPanelAdaptor
     */
    private void create_adaptors() {
	ModelToCurrentTurnPanelAdaptor MtoVct = new ModelToCurrentTurnPanelAdaptor(curr_doc, ct);
	curr_doc.addChangeListener(MtoVct);
	ModelToInformationsElementPanelAdaptor MtoVie = new ModelToInformationsElementPanelAdaptor(curr_doc, ie);
	curr_doc.addChangeListener(MtoVie);
	ModelToTurnCorectnessPanelAdaptor MtoVtc = new ModelToTurnCorectnessPanelAdaptor(curr_doc, tc);
	curr_doc.addChangeListener(MtoVtc);
	ModelToNextTurnPanelAdaptor MtoVnt = new ModelToNextTurnPanelAdaptor(curr_doc, nt);
	curr_doc.addChangeListener(MtoVnt);
	ModelToTranslationPanelAdaptor MtoDT = new ModelToTranslationPanelAdaptor(curr_doc, translation_dialog);
	curr_doc.addChangeListener(MtoDT);
	ModelToRecognizerPanelAdaptor MtoDR = new ModelToRecognizerPanelAdaptor(curr_doc, recognizer_dialog);
	curr_doc.addChangeListener(MtoDR);
	ModelToDialogActPanelAdaptor MtoDA = new ModelToDialogActPanelAdaptor(curr_doc, dialog_dialog);
	curr_doc.addChangeListener(MtoDA);
    }


    /**
     * retrieves an already opened file
     */
    private void openExistentFile(File file){
	String filename;
	// sets the curent document to the one in the vector of opened docs corresponding to the selected file
	filename = file.getName();
	for (int i = 0; i < docs.size(); i++)
	    if ((((Document)docs.elementAt(i)).file.getName()).equals(filename)) {
		curr_doc = (Document) docs.elementAt(i);
		break;
	    }
	// marks that no new document was opened
	new_document=false;
    }

    /**
     * dispalys from memory the current document
     * @see InformationElementsPanel
     * @see #DisplayCurrentTurn()
     */
    public void Redisplay() {
	//changes the informations concerning the document in the InformationsElementPanel
	ie.TN.setText("" + curr_doc.current_turn_number);
	//displays the current term of the redispalyed document
	DisplayCurrentTurn();
    }


    /**
     * exits the application and saves the non-saved information
     * @see FileSaver
     * @see Document
     */
    public void Quit() {
	int opened_docs = docs.size() - 1;
	//checks in decreasing order if there are unsaved opened docs
	while (opened_docs > -1) {
	    curr_doc= (Document) docs.elementAt(opened_docs);
	    if(!curr_doc.issaved)
		if(JOptionPane.showConfirmDialog(null,"there are unsaved changes on file "+curr_doc.file.getName()+"Will you save them?","Warning!",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION){
		    saver = new FileSaver("join",curr_doc); 
		    curr_doc.issaved=true;
		}
	    //remove the file from the opened ones
	    docs_names.removeElement(curr_doc.file.getName());
	    docs.removeElement(curr_doc);
	    opened_docs--;
	}
	//exits application
	System.exit(0);
    }




    private void deleteDocument(String name) {
	for (int i = 0; i < docs.size(); i++)
	    if ((((Document) docs.elementAt(i)).file.getName()).equals(name))
		docs.removeElementAt(i);
    }

    /**
     * close the file being in use
     * @see FileSaver
     * @see MyMenu#removeFile(String)
     * @see MyMenu#updateInClose(String)
     * @see MyMenu#addFile(String)
     * @see MyMenu#EnableMenu(boolean)
     * @see Document#fireChange()
     * @see Document#ResetValues()
     * @see #BrowseData(boolean)
     * @see #setCurrentDocument(String)
     * @see InformationElementsPanel
     */
    public void Close() {
	//number of opened file
	int opened_docs = docs.size();
	//if there is at least one opened file
	//if the current document in use is not saved
	if(curr_doc.issaved == false)
	    if(JOptionPane.showConfirmDialog(null,"there are unsaved changes on file "+curr_doc.file.getName()+". Will you save them ?","Warning!",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION) {
		//save the document to the disk
		saver = new FileSaver("join",curr_doc);
		curr_doc.issaved = true;
	    }
	//removes the name of the file from the Document menu
	menu.removeFile(curr_doc.file.getName());
	//if there are at least 2 opened docs
	if(opened_docs >= 2) {	    
	    // delete the document
	    deleteDocument((String) docs_names.elementAt(opened_docs -1));
	    // update the docs_names vector
	    docs_names.removeElementAt(opened_docs - 1);
	    opened_docs--;
	    // update the current document
	    curr_doc = setCurrentDocument((String) docs_names.elementAt(opened_docs -1));
	    //display the current turn of the new current document
	    menu.updateInClose((String) docs_names.elementAt(opened_docs -1));
	    DisplayCurrentTurn();
	    //update all the other panels related to the document
	    curr_doc.fireChange();
	}
	//if there is no document opened
	else {
	    //update the Documents menu
	    menu.addFile("no files");
	    //empty the vectors of opened docs
	    docs_names.removeAllElements();
	    docs.removeAllElements();
	    //disable any operation on the UserInterface
	    BrowseData(false);
	    EditData(false);
	    //disables operations on the menu
	    menu.EnableMenu(false);
	    menu.getMenuBar().getMenu(1).getItem(1).setSelected(false);
	    menu.getMenuBar().getMenu(1).getItem(2).setSelected(false);
	    //updates the informations concerning the document
	    curr_doc.ResetValues();
	    ie.TNs.setValue(0);
	    ie.n.setText("No Turns Loaded");
	    ie.FileName.setText("No File Open");
	    curr_doc.fireChange();
	}
    }


    /**
     * saves the document under another name
     * @param parent the parent of the pop-up window for chosing the name of the file
     * @see #SelectFile(JComponent,char)
     * @see #SaveAsNewDoc(File)
     * @see #SaveAsOldDoc(File)
     */
    public void SaveAs(JComponent parent) {
	File file;
	//if the selction window was poped-up for saving
	file = SelectFile(parent,'s');
	if ((file != (File) null)&&
	   (file.compareTo(curr_doc.file) != 0)) {
	    //if the chosed file does not exist on the disk
	    if(!file.exists()) {
		SaveAsNewDoc(file);
	    } 
	    //if the file already exists on the disk
	    else {
		String filename;
		filename = file.getName();
		//if the user wants to overwrite this file
		if (JOptionPane.showConfirmDialog(null,"the file "+filename+" already exists. do you want to overwrite it?","Warning1",JOptionPane.YES_NO_OPTION)==JOptionPane.YES_OPTION ) {
		    //if the file to be overwritten is not opened
		    if(!docs_names.contains(filename)) {
			SaveAsNewDoc(file);
		    }
		    //if the file to be overwritten is already opened
		    else 
			SaveAsOldDoc(file); 
		}
	    }
	}
    }

    /**
     * saves a new document to disk
     * @param file the full name of the document to be saved
     * @see #UpdateMenusAndVariables(File)
     * @see FileSaver
     * @see #DisplayCurrentTurn()
     */
    public void SaveAsNewDoc(File file) {
	// mark that a new document is creates
	new_document=true;
	// update the menu and the names of the file
	UpdateMenusAndVariables(file);
	// starts the saver
	saver=new FileSaver("join",curr_doc);
	// displays the current turn of the document
	DisplayCurrentTurn();
    }

    /**
     * saves the document over an already opened one
     * @param file the name of the file on which the save is done
     * @see #UpdateMenusAndVariables(File)
     * @see FileSaver
     * @see #DisplayCurrentTurn()
     */
    public void SaveAsOldDoc(File file) {
	//removes the referrences of the document to overwrite
	DeleteOldDoc(file.getName());
	//update the menu and the variables according to the new document
	UpdateMenusAndVariables(file);
	//starts the saver
	saver=new FileSaver("join",curr_doc);
	//Displays the current turn of the document
	DisplayCurrentTurn();
    }

    /**
     * updates the menu and the docs_names vector
     * @param file the name of the document
     * @see Document
     * @see MyMenu#removeFile(String)
     * @see MyMenu#addFile(String)
     */
    public void UpdateMenusAndVariables(File file) {
	//removes the old referrences to the file and file names
	menu.removeFile(curr_doc.file.getName());
	docs.remove(curr_doc);
	docs_names.remove(curr_doc.file.getName());
	//sets the new referrences to the current file
	curr_doc.file = file;
	//adds the modified document to the vectors of opened docs
	//and to the menu
	menu.addFile(curr_doc.file.getName());
	docs.add(curr_doc);
	docs_names.add(curr_doc.file.getName());
    }


    /**
     * erases all references to the old document (which will not exist 
     * anymore as another file overwrites it)
     * @see MyMenu#removeFile(String)
     */
    public void DeleteOldDoc(String filename) {
	//retrieve the old document
	for (int i = 0; i < docs.size(); i++)
	    if ((((Document)docs.elementAt(i)).file.getName()).equals(filename)) {
		//remove it's name from the Documents menu
		menu.removeFile(((Document) docs.elementAt(i)).file.getName());
		//remove it from the vectors of opened docs and docs names
		docs_names.remove(((Document) docs.elementAt(i)).file.getName());
		docs.remove((Document) docs.elementAt(i));
		break;
	    }
    }


    /**
     * starts a thread for saving if the current document is not saved
     * @see Document
     * @see FileSaver
     */
    public void Save() {
	if(curr_doc.issaved == false) {
	    saver = new FileSaver("",curr_doc);
	}
    }

    /**
     * this inner class starts a thread and saves a document to the disk
     */
    class FileSaver implements Runnable {
	Thread t;
	Document d;
	/**
	 * @param s can be join for preventing performing other operation until the save is done (e.g. exiting the appplication) or otherwise is the null string
	 * @param d the Document to be saved
	 */
	FileSaver(String s, Document d){
	    this.d=d;
	    if (s=="join"){
		t=new Thread(this);
		t.start();
		try{t.join();}
		catch(InterruptedException e){}
	    }	
	    else {
		t=new Thread(this);
		t.start();
	    }
	}
	/**
	 * after saving the document disables the Save command from the menu
	 * @see #Save(Document)
	 * @see MyMenu#setSavable(boolean)
	 */
	public void run() {
	    Save(d);
	    menu.setSavable(false);
	}

	/**
	 * Saves a document to the disk
	 * @see Document
	 * @see Judgements
	 */
	public void Save(Document d) {       
	    String data = new String("");
	    //obtain the reference to the first turn of the dialogue
	    Judgements Turn = d.getFirstTurn();
	    //retrieve the number of turns 
	    int turn_number=d.getTurnNumber();
	    //retrieve the frequency to save the turns turn 
	    int frequence_save=d.frequency_to_save;
	   
	    //the actions are synchronized on the docs vector for not allowing two savings 
	    //into the same file
	    synchronized(docs){
		try {
		    DataOutputStream out = new DataOutputStream(new FileOutputStream(d.file));
		    for (int i = 0; i < turn_number; i += frequence_save) {
			// write informations of current turn
			data = (String) Turn.turn.get("IN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<IN>"+data+"</IN>\n");
			data = (String) Turn.turn.get("OUT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<OUT>"+data+"</OUT>\n");
			data = (String) Turn.turn.get("REC");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<REC>"+data+"</REC>\n");
			data = (String) Turn.turn.get("ALI");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<ALI>"+data+"</ALI>\n");
			data = (String) Turn.turn.get("STAT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<STAT>"+data+"</STAT>\n");
			data = (String) Turn.turn.get("SYN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SYN>"+data+"</SYN>\n");
			data = (String) Turn.turn.get("DEEP");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<DEEP>"+data+"</DEEP>\n");
			data = (String) Turn.turn.get("PRE");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<PRE>"+data+"</PRE>\n");
			data = (String) Turn.turn.get("LANG");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<LANG>"+data+"</LANG>\n");
			data = (String) Turn.turn.get("DIALIN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<DIALIN>"+data+"</DIALIN>\n");
			data = (String) Turn.turn.get("DIALOUT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<DIALOUT>"+data+"</DIALOUT>\n");
			data = (String) Turn.turn.get("BEST");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<BEST>"+data+"</BEST>\n");
			data = (String) Turn.turn.get("SYNIN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SYNIN>"+data+"</SYNIN>\n");
			data = (String) Turn.turn.get("SYNOUT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SYNOUT>"+data+"</SYNOUT>\n");
			data = (String) Turn.turn.get("SEMIN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SEMIN>"+data+"</SEMIN>\n");
			data = (String) Turn.turn.get("SEMOUT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SEMOUT>"+data+"</SEMOUT>\n");
			data = (String) Turn.turn.get("MISIN");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<MISIN>"+data+"</MISIN>\n");
			data = (String) Turn.turn.get("MISOUT");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<MISOUT>"+data+"</MISOUT>\n");
			data = (String) Turn.turn.get("QUAL");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<QUAL>"+data+"</QUAL>\n");
			data = (String) Turn.turn.get("EFF");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<EFF>"+data+"</EFF>\n");
			data = (String) Turn.turn.get("SOUND");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<SOUND>"+data+"</SOUND>\n");
			data = (String) Turn.turn.get("NUMIU");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<NUMIU>"+data+"</NUMIU>\n");
			data = (String) Turn.turn.get("NESIU");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<NESIU>"+data+"</NESIU>\n");
			data = (String) Turn.turn.get("NTRNIU");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<NTRNIU>"+data+"</NTRNIU>\n");
			data = (String) Turn.turn.get("NDELIU");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<NDELIU>"+data+"</NDELIU>\n");
			data = (String) Turn.turn.get("NINNIU");
			if ((data == null)||(data.equals("null")))
			    data = new String("");
			out.writeBytes("<NINNIU>"+data+"</NINNIU>\n");
			out.writeBytes("<RECQUAL>"+recognizer_dialog.isRecognitionPerfect(Turn)+"</RECQUAL>\n");
			// insert end of turn marker
			out.writeBytes("<!----- delimiter ----->\n");
			// access next turn
			if (i < turn_number -1 )
			    Turn = (Judgements) Turn.turn.get("NEXT");	    
		    }
		    out.close();
		} catch(IOException e) {}
	    }
	}

    }

    /**
     * this inner class starts a thread for loading a document
     */
    class FileLoader implements Runnable {
	/**
	 * holds a referrence to the object that shows the progress of the loading
	 */
	Progress progres;
	Thread t;
	Document d;
	/**
	 * @param d the document to be loaded
	 * @param mainframe the parent of the progress object
	 */
	FileLoader(Document d, JFrame mainframe) {
	    this.d=d;
	    d.elem_file=0;
	    t = new Thread(this);
	    t.setPriority(t.getPriority() - 2);
	    progres = new Progress(d,mainframe);
	    t.start();
	}


	public void run() {
	    Load(d);
	}

	/**
	 * loads the document
	 * @param d the document to be loaded
	 */
	synchronized private void Load(Document d) {
	    try {
		if (d != null)		  
		    ReadFile(d);
 
	    } catch (FileNotFoundException e) {}	    
	}

	/**
	 * reads the file from the disk
	 * the method is synchronized on the vector docs for avoiding the possibility of 
	 * reading the same file in the same time
	 * @see Document#isTag(String)
	 * @see #DisplayCurrentTurn
	 * @see Judgements
	 */
	public void ReadFile(Document d) throws FileNotFoundException { 
	    String Tag;
	    synchronized(docs){
		FileReader in = new FileReader(d.file);
		StreamTokenizer tok = new StreamTokenizer(in);
		d.ResetValues();
		String currenttag = null;
		boolean tagopen = false;
		int lines = 0;
		Judgements j=d.j;
		Judgements CurrentTurn=d.CurrentTurn;
	    
		tok.resetSyntax();
		tok.whitespaceChars(60,60);
		tok.whitespaceChars(62,62);
		tok.wordChars(32,59);
		tok.wordChars(61,61);
		tok.wordChars(63,127);
		tok.eolIsSignificant(true);
	   
		try {
		    while (tok.nextToken() != tok.TT_EOF) {
			if (tok.ttype == tok.TT_WORD) {
			    // look if is a key
			    if (d.isTag(tok.sval)) {
				// if it is a key can open a tag or close it
				if (tagopen) {	    
				// if the tag is already open then it should close it
				    if (tok.sval.equals("/"+currenttag)) {
					tagopen = false;
					currenttag = null;
					d.read_quant+=tok.sval.length();
				    }
				    else {// otherwise an error has occurred
					System.out.println("Error reading file at line "+lines);
				    }
				}			    
				else {
				    currenttag = tok.sval;
				    d.read_quant+=currenttag.length();
				    if (currenttag.equals("!----- delimiter -----")) {
					j.turn.put("NEXT", (new Judgements(NO_TURNS, 1, d)));
				  
					CurrentTurn = j;
					j = (Judgements) j.turn.get("NEXT");
				   
				    }
				    else {
					tagopen = true;
				    }
				}
			    }	    
			    else {
				j.turn.put(currenttag, tok.sval);
				d.read_quant+=tok.sval.length();
			    }
			    }
			else if (tok.ttype == tok.TT_NUMBER) {
			}
			else if (tok.ttype == tok.TT_EOL) {
			    lines++;
			    d.read_quant+=8;
			}
		    }
		    progres.advance=100;
		    in.close();
		    // I have to delete the last created turn
		    CurrentTurn.turn.put("NEXT", "");
		    j = null;
		} catch (IOException e) {
		}
		DisplayCurrentTurn();
	    }
	} 

    }

    /**
     * displays the current turn of the document
     * @see Document#findTurn(int)
     */
    public void DisplayCurrentTurn() {
	ie.TNs.setMinimum(1);
	ie.TNs.setMaximum(curr_doc.turn_number);
	ie.FileName.setText("File "+curr_doc.file.getName());
	ie.n.setText("Turns loaded "+curr_doc.turn_number);
	curr_doc.findTurn(curr_doc.current_turn_number);
	if(autom_compar){
	    recognizer_dialog.compare.setEnabled(false);
	    recognizer_dialog.compare_algorithm();
	}
    }

    /**
     * displays information when no document is opened
     */
    private void DisplayNull() {
	curr_doc.fireChange();
	// Notify the slider about minimum and maximum
	ie.TNs.setMinimum(0);
	ie.TNs.setMaximum(0);
    }

    /**
     * this inner class builts a frame which shows the progress of the loading
     */
    class Progress extends JPanel implements ActionListener, Runnable {
	Document d;
	int advance=0;   
	ProgressMonitor pbar;
	/**
	 * @param d the document which is loaded
	 * @param mainframe the parent of the current frame
	 */
	public Progress(Document d,JFrame mainframe) {
	    super(true);
	    this.d=d;
	 
	    if (curr_doc.file != null)
		//the total amount of information that has to be loaded
		d.elem_file=d.file.length();	
	 
	 
	    pbar=new ProgressMonitor(mainframe,"Loading progress","Initializing...",0,100);
	    //configure and starts the progress monitor
	    pbar.setMillisToDecideToPopup(0);
	    pbar.setMillisToPopup(0);
	    // 500 is the delay between updating the progress monitor (in miliseconds)
	    timer=new javax.swing.Timer(500,this);
	    timer.start();
	}

	public void actionPerformed(ActionEvent e) {
	    SwingUtilities.invokeLater(this);
	}
	/**
	 * starts the activity of the progress monitor in a separate thread
	 */
	public void run() {
	    //if the action is canceled or the loading is finished 
	    if(pbar.isCanceled()||(advance==100)){
		pbar.close();
		timer.stop();
	    }
	    else {
		//retrieves the amount of loaded information
		advance=(int) Math.floor(((double)d.read_quant/d.elem_file)*100);
		//update the progress bar
		pbar.setProgress(advance);
		pbar.setNote(" "+advance+" % turns loaded");		
	    }
	}
    }


    /**
     * this adaptor id for the DialogActPanel
     */
    private static class ModelToDialogActPanelAdaptor implements ChangeListener {
	Document model;
	DialogActPanel view;
	
	public ModelToDialogActPanelAdaptor(Document m, DialogActPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }


    /**
     * this adaptor is for showing the translations
     */
    private static class ModelToTranslationPanelAdaptor implements ChangeListener {
	Document model;
	TranslationPanel view;
	
	public ModelToTranslationPanelAdaptor(Document m, TranslationPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }

    /**
     * this adaptor is for showing the recognition
     */
    private static class ModelToRecognizerPanelAdaptor implements ChangeListener {
	Document model;
	RecognizerPanel view;
	
	public ModelToRecognizerPanelAdaptor(Document m, RecognizerPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }


    /**
     * this adaptor is for CurrentTurnPanel controls
     */
    private static class ModelToCurrentTurnPanelAdaptor implements ChangeListener {
	Document model;
	CurrentTurnPanel view;
	
	public ModelToCurrentTurnPanelAdaptor(Document m, CurrentTurnPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }

    
    /**
     *  this adaptor is for InformationElementsPanel controls
     */
    private static class ModelToInformationsElementPanelAdaptor implements ChangeListener {
	Document model;
	InformationElementsPanel view;
	
	public ModelToInformationsElementPanelAdaptor(Document m, InformationElementsPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }

    /**
     * This adaptor is TurnCorrectnessPanel controls
     */
    private static class ModelToTurnCorectnessPanelAdaptor implements ChangeListener {
	Document model;
	TurnCorrectnessPanel view;
	
	public ModelToTurnCorectnessPanelAdaptor(Document m, TurnCorrectnessPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }

    
    /**
     * this adaptor is for NextTurnPanel controls
     */
    private static class ModelToNextTurnPanelAdaptor implements ChangeListener {
	Document model;
	NextTurnPanel view;
	
	public ModelToNextTurnPanelAdaptor(Document m, NextTurnPanel v) {
	    model = m;
	    view = v;
	}
	
	public void stateChanged(ChangeEvent e) {
	    view.changed(model.getData());
	}
    }

    /**
     * this method is called when the user changes the replace characters model in the 
     * Settings --> Options menu
     * @see ReplaceCharacters#actionPerformed(ActionEvent)
     */
    public void changed(){
	ct.changed(getCurrentDocument().getData());
	nt.changed(getCurrentDocument().getData());
	recognizer_dialog.changed(getCurrentDocument().getData());
	translation_dialog.changed(getCurrentDocument().getData());
	dialog_dialog.changed(getCurrentDocument().getData());
    }
    
}
  
